#!/usr/bin/python

import sys
from gwpy.timeseries import TimeSeries

sys.stdout.write("starting first time series \n")
    #H1_channel_lockstatus_data_1 = TimeSeries.get('H1:DMT-DC_READOUT_LOCKED',float(pw_arrival_time),float(float(pw_arrival_time) + float(options.time_after_p_wave)))
    H1_channel_lockstatus_data_1 = TimeSeries.get('H1:DMT-DC_READOUT_LOCKED',float(pw_arrival_time)-10,float(rw_arrival_time))

    print >> sys.stdout.write("data gathered")
    firstcheck = H1_channel_lockstatus_data_1[0]
    if firstcheck == 1:
        for dataoutput in H1_channel_lockstatus_data_1[1:]:
            if dataoutput == 0:
                locklosscheck1 = "Y"
                break
            else:
                locklosscheck1 = "N"
                break
    else:
        locklosscheck1 = "Z"
    print >> sys.stdout.write("first timeseries complete \n")
    #H1_channel_lockstatus_data_2 = TimeSeries.get('H1:LSC-POP_A_LF_OUT_DQ',float(pw_arrival_time),float(float(pw_arrival_time) + float(options.time_after_p_wave)))
    H1_channel_lockstatus_data_2 = TimeSeries.get('H1:LSC-POP_A_LF_OUT_DQ',float(pw_arrival_time)-10,float(rw_arrival_time))

    secondcheck = H1_channel_lockstatus_data_2[0]
    if secondcheck > 10000:
        for dataoutput in H1_channel_lockstatus_data_2:
            if dataoutput < 10:
                locklosscheck2 = "Y"
                break
            else:
                 locklosscheck2 = "N"
                 break
    else:
        locklosscheck2 = "Z"
    print >> sys.stdout.write("completed second timeseries \n")
    if locklosscheck1 == "Y" or locklosscheck2 == "Y":
        lockloss = "Y"
    elif locklosscheck1 == "N" or locklosscheck2 == "N":
        lockloss = "N"
    else:
        lockloss = "Z"



import cas
CHANNELS = {
    'H1:AMP': {
        'type': 'float',
    },
    'L1:AMP': {
        'type': 'float',
    },
    'V1:AMP': {
        'type': 'float',
    },
    'G1:AMP': {
        'type': 'float',
    },

    'MIT:AMP': {
        'type': 'float',
    },
}
prefix = 'SEISMON:'
server = cas.CaServer(prefix, CHANNELS)
server.run()



#!/usr/bin/python

import os, glob, optparse, shutil, warnings, pickle, math, copy, pickle, matplotlib
import matplotlib.pyplot as plt
import numpy as np
import scipy.signal, scipy.stats
import scipy.io
import seismon.NLNM, seismon.html
import seismon.eqmon, seismon.utils

try:
    import gwpy.time, gwpy.timeseries
    import gwpy.frequencyseries, gwpy.spectrogram
    import gwpy.plotter
except:
    print("gwpy import fails... no plotting possible.")

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__date__ = "2012/8/26"
__version__ = "0.1"

def trend(params, channel, segment):
    """@calculates spectral data for given channel and segment.

    @param params
        seismon params dictionary
    @param channel
        seismon channel structure
    @param segment
        [start,end] gps
    """

    ifo = seismon.utils.getIfo(params)

    gpsStart = segment[0]
    gpsEnd = segment[1]
   
    # set the times
    duration = np.ceil(gpsEnd-gpsStart)

    #matDir = "/home/mcoughlin/Seismon/seismicdata/STSA_BLRMS_Z_30M_100M"
    #mats = glob.glob(os.path.join(matDir,"*.mat"))

    t = []
    data = []

    #for mat in mats:
    #    mat_data = scipy.io.loadmat(mat) 
    #    if t == []:
    #        t = mat_data["timevect"]
    #        data = mat_data["datavect"]
    #    else:
    #        t = np.concatenate((t,mat_data["timevect"]))
    #        data = np.concatenate((data,mat_data["datavect"]))

    mat = "/home/mcoughlin/Seismon/seismicdata/STSA_BLRMS_Z_30M_100M_raw.mat" 
    #mat = "/home/mcoughlin/Seismon/seismicdata/STSA_BLRMS_Z_30M_100M.mat"
    mat_data = scipy.io.loadmat(mat)

    t = mat_data["time"]
    data = mat_data["data"]

    p = t[:,0].argsort() 
    t = t[p,0]
    data = data[p,0]

    sample_rate = 1.0/(t[1]-t[0])

    dataFull = gwpy.timeseries.TimeSeries(data, times=None, epoch=t[0], channel=channel.station, unit=None,sample_rate=sample_rate, name=channel.station)

    # make timeseries
    #dataFull = seismon.utils.retrieve_timeseries(params, channel, segment)
    if dataFull == []:
        return 

    dataFull = dataFull / channel.calibration
    indexes = np.where(np.isnan(dataFull.data))[0]
    meanSamples = np.mean(np.ma.masked_array(dataFull.data,np.isnan(dataFull.data)))
    for index in indexes:
        dataFull[index] = meanSamples
    dataFull -= np.mean(dataFull.data)

    if np.mean(dataFull.data) == 0.0:
        print("data only zeroes... continuing\n")
        return
    if len(dataFull.data) < 2*channel.samplef:
        print("timeseries too short for analysis... continuing\n")
        return

    if params["doEarthquakes"]:
        earthquakesDirectory = os.path.join(params["path"],"earthquakes")
        earthquakesXMLFile = os.path.join(earthquakesDirectory,"earthquakes.xml")
        attributeDics = seismon.utils.read_eqmons(earthquakesXMLFile)

    else:
        attributeDics = []

    if params["doPlots"]:

        plotDirectory = params["path"] + "/" + channel.station_underscore
        seismon.utils.mkdir(plotDirectory)        

        pngFile = os.path.join(plotDirectory,"trend.png")
        plot = gwpy.plotter.TimeSeriesPlot(figsize=[14,8])

        dataFull = dataFull.resample(16)

        plot.add_timeseries(dataFull,label="data")

        xlim = [plot.xlim[0],plot.xlim[1]]
        ylim = [plot.ylim[0],plot.ylim[1]]

        times = []
        velocities = []

        count = 0
        for attributeDic in attributeDics:

            if params["ifo"] == "IRIS":
                attributeDic = seismon.eqmon.ifotraveltimes(attributeDic, "IRIS", channel.latitude, channel.longitude)
                traveltimes = attributeDic["traveltimes"]["IRIS"]
            else:
                traveltimes = attributeDic["traveltimes"][ifo]

            Ptime = max(traveltimes["Ptimes"])
            Stime = max(traveltimes["Stimes"])
            Rtwotime = max(traveltimes["Rtwotimes"])
            RthreePointFivetime = max(traveltimes["RthreePointFivetimes"])
            Rfivetime = max(traveltimes["Rfivetimes"])

            if params["doEarthquakesVelocityMap"]:
                Rvelocitymaptimes = traveltimes["Rvelocitymaptimes"]
                Rvelocitymaptime = max(traveltimes["Rvelocitymaptimes"])
                Rvelocitymapvelocities = traveltimes["Rvelocitymapvelocities"]

            peak_velocity = traveltimes["Rfamp"][0]
            peak_velocity = peak_velocity * 1e6

            if peak_velocity > ylim[1]:
               ylim[1] = peak_velocity*1.1
            if -peak_velocity < ylim[0]:
               ylim[0] = -peak_velocity*1.1

            times.append(RthreePointFivetime)
            velocities.append(peak_velocity)

        kwargs = {"color":"k"}
        plot.add_scatter(times,velocities,label="pred. vel.",**kwargs)

        plot.ylabel = r"RMS Velocity [$\mu$m/s]"
        plot.title = channel.station.replace("_","\_")
        #plot.xlim = xlim
        #plot.ylim = ylim
        plot.add_legend(loc=1,prop={'size':10})

        #plot.axes[0].set_xscale("log")
        #plot.axes[0].set_yscale("log")

        plot.save(pngFile)
        plot.close()






import os, glob, optparse, shutil, warnings, matplotlib, pickle, math, copy, pickle, time
import numpy as np

import seismon.utils

try:
    import gwpy.time, gwpy.timeseries, gwpy.frequencyseries
    import gwpy.segments
except:
    print("gwpy import fails... no plotting possible.")

import obspy.core, obspy.signal.array_analysis
import matplotlib.pyplot as plt
import matplotlib.colorbar, matplotlib.cm, matplotlib.colors

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__date__ = "2012/8/26"
__version__ = "0.1"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def beamforming(params, segment):
    """@calculates beam forming data for given segment.

    @param params
        seismon params dictionary
    @param segment
        [start,end] gps
    """

    ifo = seismon.utils.getIfo(params)

    gpsStart = segment[0]
    gpsEnd = segment[1]

    # set the times
    duration = np.ceil(gpsEnd-gpsStart)

    st = obspy.core.Stream()

    tstart = seismon.utils.GPSToUTCDateTime(gpsStart)
    tend = seismon.utils.GPSToUTCDateTime(gpsEnd)

    for channel in params["channels"]:
        # make timeseries
        dataFull = seismon.utils.retrieve_timeseries(params, channel, segment)
        if dataFull == []:
            continue

        dataFull = dataFull / channel.calibration
        indexes = np.where(np.isnan(dataFull.data))[0]
        meanSamples = np.mean(np.ma.masked_array(dataFull.data,np.isnan(dataFull.data)))
        for index in indexes:
            dataFull[index] = meanSamples
        dataFull -= np.mean(dataFull.data)

        trace = obspy.core.Trace()
        trace.stats.network = ""
        trace.stats.station = channel.station
        trace.stats.location = ""
        trace.stats.channel = ""
        trace.stats.sampling_rate = channel.samplef
        trace.stats.delta = 1/channel.samplef
        trace.stats.calib = 1
        trace.data = np.array(dataFull.data)

        trace.stats.starttime = tstart
        trace.stats.npts = len(dataFull.data)

        trace.stats.coordinates = obspy.core.AttribDict({
            'latitude': channel.latitude,
            'longitude': channel.longitude,
            'elevation': 0.0})

        if (trace.stats.endtime < tend) or (trace.stats.starttime > tstart):
            continue
 
        st += trace

    # Execute sonic
    kwargs = dict(
        # slowness grid: X min, X max, Y min, Y max, Slow Step
        sll_x=-3.0, slm_x=3.0, sll_y=-3.0, slm_y=3.0, sl_s=0.03,
        # sliding window properties
        win_len=1.0, win_frac=0.05,
        # frequency properties
        frqlow=0.0, frqhigh=1.0, prewhiten=0,
        # restrict output
        semb_thres=-1e9, vel_thres=-1e9, timestamp='mlabday',
        #stime=tstart, etime=tstart+300,
        stime=tstart, etime=tend,
        coordsys = 'lonlat', method = 0
    )

    print("Array processing...\n")
    out = obspy.signal.array_analysis.array_processing(st, **kwargs)
    print("Finished array processing...\n")

    if params["doPlots"]:

        plotDirectory = params["path"] + "/beamforming"
        seismon.utils.mkdir(plotDirectory)

        pngFile = os.path.join(plotDirectory,"timeseries.png")

        # Plot
        labels = ['rel.power', 'abs.power', 'baz', 'slow']

        fig = plt.figure(figsize=(16, 16))
        for i, lab in enumerate(labels):
            ax = fig.add_subplot(4, 1, i + 1)
            ax.scatter(out[:, 0], out[:, i + 1], c=out[:, 1], alpha=0.6,
                   edgecolors='none')
            ax.set_ylabel(lab)
            ax.set_xlim(out[0, 0], out[-1, 0])
            ax.set_ylim(out[:, i + 1].min(), out[:, i + 1].max())

        fig.autofmt_xdate()
        fig.subplots_adjust(top=0.95, right=0.95, bottom=0.2, hspace=0)
        plt.show()
        plt.savefig(pngFile,dpi=200)
        plt.close('all')

        cmap = matplotlib.cm.hot_r

        # make output human readable, adjust backazimuth to values between 0 and 360
        t, rel_power, abs_power, baz, slow = out.T
        baz[baz < 0.0] += 360

        # choose number of fractions in plot (desirably 360 degree/N is an integer!)
        N = 30
        abins = np.arange(N + 1) * 360. / N
        sbins = np.linspace(0, 3, N + 1)

        # sum rel power in bins given by abins and sbins
        hist, baz_edges, sl_edges = np.histogram2d(baz, slow,
                bins=[abins, sbins], weights=rel_power)

        max_i,max_j = np.unravel_index(hist.argmax(), hist.shape)

        # transform to gradient
        baz_edges = baz_edges / 180 * np.pi

        pngFile = os.path.join(plotDirectory,"polar.png")

        # add polar and colorbar axes
        fig = plt.figure(figsize=(16, 16))
        cax = fig.add_axes([0.85, 0.2, 0.05, 0.5])
        ax = fig.add_axes([0.10, 0.1, 0.70, 0.7], polar=True)

        dh = abs(sl_edges[1] - sl_edges[0])
        dw = abs(baz_edges[1] - baz_edges[0])

        # circle through backazimuth
        for i, row in enumerate(hist):
            bars = ax.bar(left=(np.pi / 2 - (i + 1) * dw) * np.ones(N),
                          height=dh * np.ones(N),
                          width=dw, bottom=dh * np.arange(N),
                          color=cmap(row / hist.max()))

        ax.set_xticks([np.pi / 2, 0, 3. / 2 * np.pi, np.pi])
        ax.set_xticklabels(['N', 'E', 'S', 'W'])

        # set slowness limits
        ax.set_ylim(0, 3)
        matplotlib.colorbar.ColorbarBase(cax, cmap=cmap,
                     norm=matplotlib.colors.Normalize(vmin=hist.min(), vmax=hist.max()))

        plt.show()
        plt.savefig(pngFile,dpi=200)
        plt.close('all')

def strainz(params, segment):
    """@calculates strain for given segment.

    @param params
        seismon params dictionary
    @param segment
        [start,end] gps
    """

    ifo = seismon.utils.getIfo(params)

    gpsStart = segment[0]
    gpsEnd = segment[1]

    # set the times
    duration = np.ceil(gpsEnd-gpsStart)

    dataAll = []

    for channel in params["channels"]:
        # make timeseries
        dataFull = seismon.utils.retrieve_timeseries(params, channel, segment)
        if dataFull == []:
            continue

        dataFull = dataFull / channel.calibration
        indexes = np.where(np.isnan(dataFull.data))[0]
        meanSamples = np.mean(np.ma.masked_array(dataFull.data,np.isnan(dataFull.data)))
        for index in indexes:
            dataFull[index] = meanSamples
        dataFull -= np.mean(dataFull.data)

        if np.mean(dataFull.data) == 0.0:
            print("data only zeroes... continuing\n")
            continue
        if len(dataFull.data) < 2*channel.samplef:
            print("timeseries too short for analysis... continuing\n")
            continue

        cutoff = 0.3
        dataFull = dataFull.lowpass(cutoff, amplitude=0.9, order=3, method='scipy')
        dataFull = dataFull.resample(16)

        dataAll.append(dataFull)

    ts1 = []
    ts2 = []
    ts3 = []
    for dataFull in dataAll:
        if "SEISX" in dataFull.channel.name:
            if ts1 == []:
                ts1 = dataFull.data
            else:
                ts1 = np.vstack([ts1,dataFull.data])
        if "SEISY" in dataFull.channel.name:
            if ts2 == []:
                ts2 = dataFull.data
            else:
                ts2 = np.vstack([ts2,dataFull.data])
        if "SEISZ" in dataFull.channel.name:
            if ts3 == []:
                ts3 = dataFull.data
            else:
                ts3 = np.vstack([ts3,dataFull.data])

    tt = np.array(dataFull.times)

    ts1 = ts1.T
    ts2 = ts2.T
    ts3 = ts3.T

    Nsamples,Na = ts1.shape
    subarray = np.arange(Na)
    vp = 8.0
    vs = 4.0
    array_coords = np.array([[0,0,0],[4000,0,0],[0,4000,0]])

    sigmau = np.zeros((Na,3))
    for i in range(Na):
        sigmau[i,0] = np.std(ts1[i,:60*channel.samplef])
        sigmau[i,1] = np.std(ts2[i,:60*channel.samplef])
        sigmau[i,2] = np.std(ts3[i,:60*channel.samplef])

    out_dic = obspy.signal.array_analysis.array_rotation_strain(subarray, ts1, ts2, ts3, vp, vs, array_coords, sigmau)

    if params["doPlots"]:

        plotDirectory = params["path"] + "/strainz"
        seismon.utils.mkdir(plotDirectory)

        pngFile = os.path.join(plotDirectory,"strainz.png")

        # Plot
        labels = ['Shear strain [1/s]', 'Dilation [1/s]', 'x1 rotation [rad/s]', 
            'x2 rotation [rad/s]','x3 rotation [rad/s]','Misfit']

        fig = plt.figure(figsize=(32, 24))
        for i, lab in enumerate(labels):
            ax = fig.add_subplot(6, 1, i + 1)
            data = gwpy.timeseries.TimeSeries(data, epoch=tt[0], sample_rate=1.0/(tt[1]-tt[0]))
            if i == 0:
                data = out_dic["ts_s"]
                ax.plot(tt,out_dic["ts_s"])
            elif i == 1:
                data = out_dic["ts_d"]
                ax.plot(tt,out_dic["ts_d"])
            elif i == 2:
                data = out_dic["ts_w1"]
                ax.plot(tt,out_dic["ts_w1"])
            elif i == 3:
                data = out_dic["ts_w2"]
                ax.plot(tt,out_dic["ts_w2"])
            elif i == 4:
                data = out_dic["ts_w3"]
                ax.plot(tt,out_dic["ts_w3"])
            elif i == 5:
                data = out_dic["ts_M"]
                ax.plot(tt,out_dic["ts_M"])
            data = gwpy.timeseries.TimeSeries(data, epoch=tt[0], sample_rate=1.0/(tt[1]-tt[0]))
            ax.set_ylabel(lab)
            #ax.set_xlim(out[0, 0], out[-1, 0])
            #ax.set_ylim(out[:, i + 1].min(), out[:, i + 1].max())

        fig.autofmt_xdate()
        fig.subplots_adjust(top=0.95, right=0.95, bottom=0.2, hspace=0)
        plt.xlabel('Time [s]')
        plt.show()
        plt.savefig(pngFile,dpi=200)
        plt.close('all')




"""
models.py : Primary script of SEISMON framework to make ground motion predictions [um/s] 
at the GW detectors from initial USGS earthquake parameters acquired using the 
USGS PDL client.

# Make sure USGS PDL client is running

# Specify correct path & settings in the input/config.yaml file

# To Initialize the database:
python models.py --init_db

# To purge all the events 
python models.py --purge

# Normal execution (once thee database is configured)
python models.py --config input/config.yaml

 

"""

from datetime import datetime, date
import simplejson as json
import enum
import os
import glob
import time
import copy
import configparser
import pickle

from astropy import table
from astropy import coordinates
from astropy import units as u
from astropy.time import Time, TimeDelta
import pkg_resources
import numpy as np
import pandas as pd
from os.path import basename,splitext 

import sqlalchemy as sa
from sqlalchemy.ext.associationproxy import association_proxy
from sqlalchemy.ext.declarative import declarative_base, declared_attr
from sqlalchemy.orm import sessionmaker, scoped_session, relationship
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy import func
from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy import create_engine  
#from arrow.arrow import Arrow

from obspy.geodetics.base import gps2dist_azimuth
from obspy.taup import TauPyModel

import seismon
from seismon import (eqmon, utils)
from seismon.config import app

from flask_login.mixins import UserMixin
from flask_sqlalchemy import SQLAlchemy




#-------------Initialize Global Variables (dont, change modified inside functions)---------------
# 
FLAG_is_llo_gpr_model_loaded = 0 # 1 if model for eqmon.gprPrediction loaded
FLAG_is_lho_gpr_model_loaded = 0
#-----------------------------------------



db = SQLAlchemy(app)

DBSession = scoped_session(sessionmaker())
EXECUTEMANY_PAGESIZE = 50000
utcnow = func.timezone('UTC', func.current_timestamp())

data_types = {
    int: 'int',
    float: 'float',
    bool: 'bool',
    dict: 'dict',
    str: 'str',
    list: 'list'
    }


class Encoder(json.JSONEncoder):
    """Extends json.JSONEncoder with additional capabilities/configurations."""
    def default(self, o):
        if isinstance(o, (datetime, Arrow, date)):
            return o.isoformat()

        elif isinstance(o, bytes):
            return o.decode('utf-8')

        elif hasattr(o, '__table__'):  # SQLAlchemy model
            return o.to_dict()

        elif o is int:
            return 'int'

        elif o is float:
            return 'float'

        elif type(o).__name__ == 'ndarray': # avoid numpy import
            return o.tolist()

        elif type(o).__name__ == 'DataFrame':  # avoid pandas import
            o.columns = o.columns.droplevel('channel')  # flatten MultiIndex
            return o.to_dict(orient='index')

        elif type(o) is type and o in data_types:
            return data_types[o]

        return json.JSONEncoder.default(self, o)

def to_json(obj):
    return json.dumps(obj, cls=Encoder, indent=2, ignore_nan=True)

class BaseMixin(object):
    query = DBSession.query_property()
    id = sa.Column(sa.Integer, primary_key=True)
    created_at = sa.Column(sa.DateTime, nullable=False, default=utcnow)
    modified = sa.Column(sa.DateTime, default=utcnow, onupdate=utcnow,
                         nullable=False)

    @declared_attr
    def __tablename__(cls):
        return cls.__name__.lower() + 's'

    __mapper_args__ = {'confirm_deleted_rows': False}

    def __str__(self):
        return to_json(self)

    def __repr__(self):
        attr_list = [f"{c.name}={getattr(self, c.name)}"
                     for c in self.__table__.columns]
        return f"<{type(self).__name__}({', '.join(attr_list)})>"

    def to_dict(self):
        if sa.inspection.inspect(self).expired:
            DBSession().refresh(self)
        return {k: v for k, v in self.__dict__.items() if not k.startswith('_')}

    @classmethod
    def get_if_owned_by(cls, ident, user, options=[]):
        obj = cls.query.options(options).get(ident)

        if obj is not None and not obj.is_owned_by(user):
            raise AccessError('Insufficient permissions.')

        return obj

    def is_owned_by(self, user):
        raise NotImplementedError("Ownership logic is application-specific")

    @classmethod
    def create_or_get(cls, id):
        obj = cls.query.get(id)
        if obj is not None:
            return obj
        else:
            return cls(id=id)

Base = declarative_base(cls=BaseMixin)

# The db has to be initialized later; this is done by the app itself
# See `app_server.py`
def init_db(user, database, password=None, host=None, port=None):
    url = 'postgresql://{}:{}@{}:{}/{}'
    url = url.format(user, password or '', host or '', port or '', database)

    conn = sa.create_engine(url, client_encoding='utf8')
#                            executemany_mode='values',
#                            executemany_values_page_size=EXECUTEMANY_PAGESIZE)

    DBSession.configure(bind=conn)
    Base.metadata.bind = conn

    return conn


class Earthquake(Base):
    """Earthquake information"""

    event_id = sa.Column(
        sa.String,
        unique=True,
        nullable=False,
        comment='Earthquake ID')

    lat = sa.Column(
        sa.Float,
        nullable=False,
        comment='Latitude')

    lon = sa.Column(
        sa.Float,
        nullable=False,
        comment='Longitude')

    depth = sa.Column(
        sa.Float,
        nullable=False,
        comment='Depth')

    magnitude = sa.Column(
        sa.Float,
        nullable=False,
        comment='Magnitude',
        index=True)

    date = sa.Column(
        sa.DateTime,
        nullable=False,
        comment='UTC event timestamp',
        index=True)

    sent = sa.Column(
        sa.DateTime,
        nullable=False,
        comment='UTC sent timestamp')

    predictions = relationship(lambda: Prediction)


class Ifo(Base):
    """Detector information"""

    id = sa.Column(sa.Integer, primary_key=True)

    ifo = sa.Column(
        sa.String,
        unique=True,
        nullable=False,
        comment='Detector name')

    lat = sa.Column(
        sa.Float,
        nullable=False,
        comment='Latitude')

    lon = sa.Column(
        sa.Float,
        nullable=False,
        comment='Longitude')

    predictions = relationship(lambda: Prediction)


class Prediction(Base):
    """Prediction information"""

    id = sa.Column(sa.Integer, primary_key=True)

    event_id = sa.Column(
        sa.String,
        sa.ForeignKey(Earthquake.event_id),
        nullable=False,
        comment='Earthquake ID')

    ifo = sa.Column(
        sa.String,
        sa.ForeignKey(Ifo.ifo),
        nullable=False,
        comment='Detector name')        

    magnitude = sa.Column(
        sa.Float,
        nullable=False,
        comment='Magnitude')

    depth = sa.Column(
        sa.Float,
        nullable=False,
        comment='Depth')        

    lat = sa.Column(
        sa.Float,
        nullable=False,
        comment='Latitude')    

    lon = sa.Column(
        sa.Float,
        nullable=False,
        comment='Longitude')                
                                                                 

    d = sa.Column(
        sa.Float,
        nullable=False,
        comment='Distance [km]')

    p = sa.Column(
        sa.DateTime,
        nullable=False,
        comment='P-wave time')

    s = sa.Column(
        sa.DateTime,
        nullable=False,
        comment='S-wave time')

    r2p0 = sa.Column(
           sa.DateTime,
           nullable=False,
           comment='R-2.0 km/s-wave time')

    r3p5 = sa.Column(
           sa.DateTime,
           nullable=False,
           comment='R-3.5 km/s-wave time')

    r5p0 = sa.Column(
           sa.DateTime,
           nullable=False,
           comment='R-5.0 km/s-wave time')

    rfamp = sa.Column(
            sa.Float,
            nullable=False,
            comment='ML-based Earthquake amplitude predictions [um/s]')
    
    rfamp_std = sa.Column(
        sa.Float,
        nullable=False,
        comment='std for ML-based Earthquake amplitude predictions [um/s]')
    
    rfamp_linear = sa.Column(
            sa.Float,
            nullable=False,
            comment='PowerLawFit-based Earthquake amplitude predictions [um/s]')    

    #(added by NM on 04/10/21)
    rfamp_measured = sa.Column(
            sa.Float,
            nullable=False,
            comment='Earthquake amplitude measured [um/s]')

    lockloss = sa.Column(
               sa.INT,
               nullable=False,
               comment='Earthquake lockloss prediction')


class llo_catalogue(Base):
    """LLO catalogue information"""

    event_id = sa.Column(
        sa.String,
        #sa.ForeignKey(Earthquake.event_id),
        nullable=False,
        comment='Earthquake ID')

    time= sa.Column(
            sa.String,
            nullable=False,
            comment='time')                      

    latitude= sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ latitude')   

    longitude= sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ longitude')                                   

    depth = sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ depth')    

    mag= sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ Magnitude')               

    place = sa.Column(
        sa.String,
        nullable=False,
        comment='EQ Place')   


    SNR = sa.Column(
            sa.Float,
            nullable=False,
            comment='Signal-to-Noise Ratio')          


    peak_data_um_mean_subtracted = sa.Column(
            sa.Float,
            nullable=False,
            comment='Earthquake amplitude predictions [um/s]')      


class lho_catalogue(Base):
    """LLO catalogue information"""

    event_id = sa.Column(
        sa.String,
        #sa.ForeignKey(Earthquake.event_id),
        nullable=False,
        comment='Earthquake ID')

    time= sa.Column(
            sa.String,
            nullable=False,
            comment='time')                      

    latitude= sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ latitude')   

    longitude= sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ longitude')                                   

    depth = sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ depth')    

    mag= sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ Magnitude')               

    place = sa.Column(
        sa.String,
        nullable=False,
        comment='EQ Place')   


    SNR = sa.Column(
            sa.Float,
            nullable=False,
            comment='Signal-to-Noise Ratio')          


    peak_data_um_mean_subtracted = sa.Column(
            sa.Float,
            nullable=False,
            comment='Earthquake amplitude predictions [um/s]')     

class virgo_catalogue(Base):
    """LLO catalogue information"""

    event_id = sa.Column(
        sa.String,
        #sa.ForeignKey(Earthquake.event_id),
        nullable=False,
        comment='Earthquake ID')

    time= sa.Column(
            sa.String,
            nullable=False,
            comment='time')                      

    latitude= sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ latitude')   

    longitude= sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ longitude')                                   

    depth = sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ depth')    

    mag= sa.Column(
            sa.Float,
            nullable=False,
            comment='EQ Magnitude')               

    place = sa.Column(
        sa.String,
        nullable=False,
        comment='EQ Place')   


    SNR = sa.Column(
            sa.Float,
            nullable=False,
            comment='Signal-to-Noise Ratio')          


    peak_data_um_mean_subtracted = sa.Column(
            sa.Float,
            nullable=False,
            comment='Earthquake amplitude predictions [um/s]')                 


def compute_predictions(earthquake, ifo):

    Dist, Ptime, Stime, Rtwotime, RthreePointFivetime, Rfivetime = compute_traveltimes(earthquake, ifo) 
    Rfamp, Lockloss,Rfamp_powerLawFit, Rfamp_std = compute_amplitudes(earthquake, ifo)
    
    # make sure Rfamp etc is in float 
    Rfamp = np.float(Rfamp)
    Rfamp_std = np.float(Rfamp_std)
    Rfamp_powerLawFit = np.float(Rfamp_powerLawFit)

    #PRINT: only LLO/LHO
    if ifo.ifo.lower()=='llo' or ifo.ifo.lower()=='lho' :
        print('Prediction for event: {} with mag: {:0.2f} at IFO: {}'.
            format(earthquake.event_id,earthquake.magnitude,ifo.ifo))
        print('....... Linear-prediction:{:0.2f} um/s'.format(Rfamp_powerLawFit))
        print('....... ML-prediction:{:0.2f} um/s'.format(Rfamp))
        print('-----------------------------------------------')
   


    DBSession().merge(Prediction(event_id=earthquake.event_id,
                                 ifo=ifo.ifo,
                                 magnitude=earthquake.magnitude,
                                 depth=earthquake.depth,
                                 lat=earthquake.lat,
                                 lon=earthquake.lon,
				                 d=Dist,
                                 p=Ptime,
                                 s=Stime,
                                 r2p0=Rtwotime,
                                 r3p5=RthreePointFivetime,
                                 r5p0=Rfivetime,
                                 rfamp=Rfamp,
                                 rfamp_std=Rfamp_std,
                                 rfamp_linear=Rfamp_powerLawFit,
                                 rfamp_measured=-1,
                                 lockloss=int(Lockloss)))
    DBSession().commit()


def compute_traveltimes(earthquake, ifo):

    seismon_path = os.path.dirname(seismon.__file__)
    script_path = os.path.join(seismon_path,'input')

    depth = earthquake.depth
    eqtime = Time(earthquake.date, format='datetime')
    eqlat = earthquake.lat
    eqlon = earthquake.lon
    ifolat = ifo.lat
    ifolon = ifo.lon

    distance,fwd,back = gps2dist_azimuth(eqlat,
                                         eqlon,
                                         ifolat,
                                         ifolon)
    Dist = distance/1000
    degree = (distance/6370000)*(180/np.pi)

    model = TauPyModel(model="iasp91")

    Rtwotime = eqtime+TimeDelta(distance/2000.0 * u.s)
    RthreePointFivetime = eqtime+TimeDelta(distance/3500.0 * u.s)
    Rfivetime = eqtime+TimeDelta(distance/5000.0 * u.s)

    try:
        arrivals = model.get_travel_times(source_depth_in_km=depth,
                                          distance_in_degree=degree)

        Ptime = -1
        Stime = -1
        for phase in arrivals:
            if Ptime == -1 and phase.name.lower()[0] == "p":
                Ptime = eqtime+TimeDelta(phase.time * u.s)
            if Stime == -1 and phase.name.lower()[0] == "s":
                Stime = eqtime+TimeDelta(phase.time * u.s)
    except:
        Ptime, Stime = Rtwotime, Rtwotime 

    return Dist, Ptime.datetime, Stime.datetime, Rtwotime.datetime, RthreePointFivetime.datetime, Rfivetime.datetime


def compute_amplitudes(earthquake, ifo):
    
    # global variables (that gets modified globally)
    global FLAG_is_llo_gpr_model_loaded 
    global FLAG_is_lho_gpr_model_loaded

    global llo_model
    global lho_model
    

    seismon_path = os.path.dirname(seismon.__file__)
    script_path = os.path.join(seismon_path,'input')

    depth = earthquake.depth
    eqtime = Time(earthquake.date, format='datetime')
    eqlat = earthquake.lat
    eqlon = earthquake.lon
    mag = earthquake.magnitude


   
    '''
    # OLD-SEISMON

    ifolat = ifo.lat
    ifolon = ifo.lon

    if ifo.ifo == "LLO":
        trainFile = os.path.join(script_path,'LLO_processed_USGS_global_EQ_catalogue.csv')
        catalogue_name = 'llo_catalogues'
    elif ifo.ifo == "Virgo":
        trainFile = os.path.join(script_path,'LHO_processed_USGS_global_EQ_catalogue.csv')
        catalogue_name = 'virgo_catalogues'
    else:
        trainFile = os.path.join(script_path,'LHO_processed_USGS_global_EQ_catalogue.csv')
        catalogue_name = 'lho_catalogues'


    # Read from the SQL database 
    try:
        trainData = pd.read_sql_query('select * from public.{}'.format(catalogue_name),con=conn)  
        if trainData.empty:
            print('trainData DataFrame is empty! Update EQ Catalogue Database !!!')
            print('Reverting back to CSV based data fetching.')
            trainData = pd.read_csv(trainFile)

    except Exception as Excep: 
        print(Excep)
        print('Error occured. Could not connect to database. Reverting back to CSV based data fetching.')
        trainData = pd.read_csv(trainFile)

    thresh=0.1 #used to limit the geographical extend (latitude & longitude) to search around the current event
    predictor='peak_data_um_mean_subtracted'
    predicted_peak_amplitude,LocklossTag,Rfamp_sigma,LocklossTag_sigma,TD = eqmon.make_prediction(trainData,
                    eqlat,
                    eqlon,
                    mag,
                    depth,
                    ifolat,
                    ifolon,
                    thresh,predictor,locklossMotionThresh)
    #print(TD)                    
    '''
    

    # call powerLawFit for comparison
    Y_pred_powerLawFit,Y_pred_st_powerLawFit = eqmon.powerLawFit(ifo,eqlat,eqlon,mag,depth)   

    # NEW GPR-SEISMON
    # load model using pickle

    #TOFIX: (currently  GEO,Virgo uses LHO GPR model)
    if ifo.ifo.upper()=="LLO": 
        model_name = "gpr_model_LLO.dump"
    else:
        model_name = "gpr_model_LHO.dump"

    model_fullname = os.path.join(seismon_path,model_path,model_name)

    # LOAD IFO Specific GPR Models (just once)
    if ifo.ifo.lower() == "llo":  
        # LLO      
        if FLAG_is_llo_gpr_model_loaded==0:
            llo_model= pickle.load(open(model_fullname,"rb"))
            model=llo_model
            FLAG_is_llo_gpr_model_loaded=1
        else:
            model=llo_model
    else:     
        # CHECK: Currently lho_model used for lho/virgo/geo
        # LHO  
        if FLAG_is_lho_gpr_model_loaded==0:
            lho_model= pickle.load(open(model_fullname,"rb"))
            model=lho_model
            FLAG_is_lho_gpr_model_loaded=1
        else:
            model=lho_model              

    # call gprPredict 
    Y_pred,Y_pred_std,model = eqmon.gprPredict(0,model_fullname,model,ifo,eqlat,eqlon,mag,depth)
    
    '''
    # CHECK: Since GPR-model is only trained onn events with Mag>5.5. Set others ->0
    if mag < min_eq_magnitude:
        print('magnitude ({:0.2f}) less than min_eq_magnitude'.format(mag))
        print('Setting GPR-predicted amplitude to zero')
        Y_pred=0
        Y_pred_std=0
    '''


    
    
    # set LocklossTag FLAG
    if Y_pred > locklossMotionThresh*1e-6: # Y_pred is in um/s, while locklossMotionThresh is in m/s
        LocklossTag = 1
    else:
        LocklossTag = 0
    predicted_peak_amplitude = Y_pred
    return predicted_peak_amplitude, LocklossTag, Y_pred_powerLawFit,Y_pred_std





def ingest_ifos():

    ifos = {"LHO": {'Latitude': 46.6475, 'Longitude': -119.5986},
            "LLO": {'Latitude': 30.4986, 'Longitude': -90.7483},
            "GEO": {'Latitude': 52.246944, 'Longitude': 9.80833},
            "VIRGO": {'Latitude': 43.631389, 'Longitude': 10.505},
            "KAGRA": {'Latitude': 36.4119, 'Longitude': 137.3058}}

    for det in ifos.keys():
        DBSession().merge(Ifo(ifo=det,
                              lat=ifos[det]["Latitude"],
                              lon=ifos[det]["Longitude"]))
    DBSession().commit()

def ingest_earthquakes(config, lookback, repeat=False):

# convert lookback to TimeDelta
    lookbackTD = TimeDelta(lookback,format='jd')

    folders = glob.glob(os.path.join(config["pdlcient"]["directory"],"*"))
    for folder in folders:
        folderSplit = folder.split("/")
        eventName = folderSplit[len(folderSplit) - 1]
        dataFolder = os.path.join(folder, eventName[0:2])
        timeFolders = glob.glob(os.path.join(dataFolder,"*"))
        timeFolders = sorted(timeFolders)

        if timeFolders == []:
            continue

        for timeFolder in timeFolders:
            
            attributeDic = []
            eqxmlfile = os.path.join(timeFolder,"eqxml.xml")
            quakemlfile = os.path.join(timeFolder,"quakeml.xml")

            if not repeat:
                if os.path.isfile(os.path.join(timeFolder,"eqxml.txt")):
                    continue

            f = open(os.path.join(timeFolder,"eqxml.txt"),"w")
            f.write("Done")
            f.close()

            if os.path.isfile(eqxmlfile):
                attributeDic = eqmon.read_eqxml(eqxmlfile,eventName)
            elif os.path.isfile(quakemlfile):
                attributeDic = eqmon.read_quakeml(quakemlfile,eventName)

            if attributeDic == []:
                continue

            if (not "GPS" in attributeDic) or (not "Magnitude" in attributeDic):
                continue

            date = Time(attributeDic["Time"], format='isot', scale='utc')

            #(modified by NM on 03/10/21 to skip >> KeyError: 'Sent' )
            try:
                sent = Time(attributeDic["Sent"], format='isot', scale='utc')
            except:
                sent = Time(attributeDic["Time"], format='isot', scale='utc')



            if Time.now() - date > lookbackTD: continue

            eqs = Earthquake.query.filter_by(event_id=attributeDic["eventName"]).all()
            if len(eqs) > 0: continue

            DBSession().merge(Earthquake(depth=attributeDic["Depth"],
                                         lat=attributeDic["Latitude"],
                                         lon=attributeDic["Longitude"],
                                         event_id=attributeDic["eventName"],
                                         magnitude=attributeDic["Magnitude"],
                                         date=date.datetime,
                                         sent=sent.datetime))


                                         
            print('Ingested event: %s' % attributeDic["eventName"])
            DBSession().commit()


def run_seismon(purge=False, init_db=False):

    if purge:
        sys_command = "find %s/* -type d -mtime +7 -exec rm -rf {} \;" % config["pdlcient"]["directory"]
        os.system(sys_command)

    if init_db:
        
        # Upload Past EQ events from CSV to the Database
        seismon_path = os.path.dirname(seismon.__file__)
        testpath = os.path.join(seismon_path,'tests')
        dataframe2database = os.path.join(testpath,'test_upload_pandas_table_to_database.py')
        os.system('python {}'.format(dataframe2database))

        ingest_earthquakes(config, args.lookback, repeat=True)
    else:
        ingest_earthquakes(config, args.lookback)

    ifos = Ifo.query.all()
    eqs = Earthquake.query.all()

    for eq in eqs:
        for det in ifos:
            preds = Prediction.query.filter_by(event_id=eq.event_id,
                                               ifo=det.ifo).all()

            #[print(val) for val in preds]                                              

            # check if event is already processed & if the event magnitude is higher than the specified threshold(modified by NM, 01/10/21) 
            if len(preds) == 0 and eq.magnitude >= float(config['database']['min_eq_magnitude']) :
                compute_predictions(eq, det)

                # (added by NM on 02/10/21) Sent event to Caltech machine                
                # (added by NM on 03/10/21) KeyError for: sent = Time(attributeDic["Sent"], format='isot', scale='utc')
                try:
                    mydict={'event_id':eq.event_id,'lat':eq.lat,'lon':eq.lon,'magnitude':eq.magnitude,'depth':eq.depth,'event_time':str(eq.date),'sent':str(eq.sent),'created_at':str(eq.created_at),'modified':str(eq.modified)}
                except:
                    mydict={'event_id':eq.event_id,'lat':eq.lat,'lon':eq.lon,'magnitude':eq.magnitude,'depth':eq.depth,'event_time':str(eq.date),'sent':str(eq.date),'created_at':str(eq.created_at),'modified':str(eq.modified)}


                
                event_filename='./tests/new_events/{0}.csv'.format(eq.event_id)
                pd.DataFrame([mydict]).to_csv(event_filename, index=False)    
                syscmd='scp {0} {1}@{2}:{3}'.format(event_filename,ldg_uname,ldg_cluster,ldg_cluster_pdl_client_event_folder)
                # only sent once while looping over ifos 
                if FLAG_send_pdl_event_to_ldg == 1:
                    if det.ifo=="LHO":
                        try:
                            print('attempting to send the new event file {0} to Caltech machine'.format(event_filename))
                            os.system(syscmd)
                            print('File sent.')
                        except:
                            print('unable to send the file to Caltech machine')
                            pass


            # Print Predictions, for debugging purpose
            '''
            try:   #(modified by NM, 03/02/22) 
                # uncomment below to print predictions
                preds = Prediction.query.filter_by(event_id=eq.event_id,ifo=det.ifo).all()
                [print(val) for val in preds]

            except:
                pass
           '''



if __name__ == "__main__":

    from argparse import ArgumentParser
    parser = ArgumentParser()

    parser.add_argument('-i', '--init_db', action='store_true', default=False)
    parser.add_argument('-p', '--purge', action='store_true', default=False)
    parser.add_argument('-C', '--config', default='input/config.yaml')
    parser.add_argument('-l', '--lookback', default=7, help='lookback in days')
    parser.add_argument("-d", "--debug", action="store_true", default=False)

    args = parser.parse_args()

    if not os.path.isfile(args.config):
        print('Missing config file: %s' % args.config)
        exit(1)

    config = configparser.ConfigParser()
    config.read(args.config)

    # Fetch from config
    # [database]
    min_eq_magnitude = float(config['database']['min_eq_magnitude'])
    # [eq_prediction]
    locklossMotionThresh =  float(config['eq_prediction']['locklossMotionThresh'])
    model_path = config['eq_prediction']['model_path']
    # [ldg_params]
    FLAG_send_pdl_event_to_ldg= int(config['ldg_params']['FLAG_send_pdl_event_to_ldg'])
    ldg_uname = config['ldg_params']['ldg_uname']
    ldg_cluster = config['ldg_params']['ldg_cluster']
    ldg_cluster_pdl_client_event_folder = config['ldg_params']['ldg_cluster_pdl_client_event_folder']

    conn = init_db(config['database']['user'],
                   config['database']['database'],
                   password=config['database']['password'],
                   host=config['database']['host'],
                   port=config['database']['port'])

    if args.init_db:
        print(f'Creating tables on database {conn.url.database}')
        Base.metadata.drop_all()
        Base.metadata.create_all()

        print('Refreshed tables:')
        for m in Base.metadata.tables:
            print(f' - {m}')
        ingest_ifos()

    if args.debug:
        run_seismon(purge=args.purge, init_db=args.init_db)
        exit(0)

    while True:
        #try:
        print('Looking for some earthquakes to analyze!')
        run_seismon(purge=args.purge, init_db=args.init_db)
        #except:
        #    pass
        time.sleep( int(config['eq_prediction']['sleep_time_secs']) )



#!/usr/bin/python

"""
%prog

Michael Coughlin (coughlim@carleton.edu)

This program checks for earthquakes.

"""

import os, time, glob, matplotlib, math
import numpy as np
import scipy.optimize
try:
    from mpl_toolkits.basemap import Basemap
except:
    print("Basemap import failed ...")
matplotlib.use("AGG")
matplotlib.rcParams.update({'font.size': 18})
from matplotlib import pyplot as plt
from matplotlib import cm

import seismon.eqmon

try:
    import gwpy.time, gwpy.timeseries
    import gwpy.frequencyseries, gwpy.spectrogram
except:
    print("gwpy import fails... no plotting possible.")

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def restimates(params,attributeDics,plotName):
    """@restimates plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param plotName
        name of plot    
    """

    ifo = seismon.utils.getIfo(params)

    gps = []
    magnitudes = []
    for attributeDic in attributeDics:

        if not "Arbitrary" in attributeDic["traveltimes"]:
            continue

        if params["ifo"] == "IRIS":
            distances = []
            for channel in params["channels"]:
                attributeDic = seismon.eqmon.ifotraveltimes_loc(attributeDic, "IRIS", channel.latitude, channel.longitude)
                traveltimes = attributeDic["traveltimes"]["IRIS"]
                distances.append(traveltimes["Distances"][0])
            index = np.argmax(distances)
            channel = params["channels"][index]
            attributeDic = seismon.eqmon.ifotraveltimes_loc(attributeDic, "IRIS", channel.latitude, channel.longitude)
            traveltimes = attributeDic["traveltimes"]["IRIS"]

        else:
            traveltimes = attributeDic["traveltimes"][ifo]


        if "Restimate" not in traveltimes:
            continue

        travel_time = traveltimes["Restimate"] - traveltimes["RthreePointFivetimes"][-1]

        gps.append(travel_time)
        magnitudes.append(attributeDic["Magnitude"])

    if magnitudes == []:
        return

    gps = np.array(gps)
    magnitudes = np.array(magnitudes)

    plt.figure()
    plt.plot(magnitudes,gps, 'k*')
    plt.xlim([min(magnitudes)-0.5, max(magnitudes)+0.5])
    plt.xlabel('Magnitude')
    plt.ylabel("\Delta T")
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def timedelay_plot(params,data,plotName):
    """@restimates plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param plotName
        name of plot
    """

    index = []
    amp = []
    RthreePointFivetimes = []
    Ptimes = []
    distance = []
    magnitude = []
    depth = []

    for key in data["channels"].iterkeys():

        label = key.replace("_","\_")

        channel_data = data["channels"][key]["earthquakes"]

        for i in range(len(channel_data["RthreePointFivetimes"])):

            RthreePointFivetime = channel_data["RthreePointFivetimes"][i]
            Ptime = channel_data["Ptimes"][i]
            #ampMax = channel_data["ampMax"][i]
            
            amp.append(channel_data["ampMax"][i])
            distance.append(channel_data["distance"][i])
            depth.append(channel_data["depth"][i])
            magnitude.append(channel_data["magnitude"][i])
            RthreePointFivetimes.append(RthreePointFivetime)
            Ptimes.append(Ptime)
    
    if magnitude == []:
        return

    magnitude = np.array(magnitude)
    amp = np.array(amp)
    distance = np.array(distance)
    depth = np.array(depth)
    Ptimes = np.array(Ptimes)
    RthreePointFivetimes = np.array(RthreePointFivetimes)
    diffs = RthreePointFivetimes - Ptimes

    plt.figure()
    plt.loglog(diffs,amp,'*')
    #plt.xlim([min(magnitudes)-0.5, max(magnitudes)+0.5])
    plt.xlabel('Delay between P and R-wave arrivals [s]')
    plt.ylabel('Peak ground velocity [m/s]')
    #plt.ylabel("\Delta T")
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def timedelay_distance_plot(params,data,plotName):
    """@restimates plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param plotName
        name of plot
    """

    index = []
    amp = []
    RthreePointFivetimes = []
    Ptimes = []
    distance = []
    magnitude = []
    depth = []

    for key in data["channels"].iterkeys():

        label = key.replace("_","\_")

        channel_data = data["channels"][key]["earthquakes"]

        for i in range(len(channel_data["RthreePointFivetimes"])):

            RthreePointFivetime = channel_data["RthreePointFivetimes"][i]
            Ptime = channel_data["Ptimes"][i]
            #ampMax = channel_data["ampMax"][i]

            amp.append(channel_data["ampMax"][i])
            distance.append(channel_data["distance"][i])
            depth.append(channel_data["depth"][i])
            magnitude.append(channel_data["magnitude"][i])
            RthreePointFivetimes.append(RthreePointFivetime)
            Ptimes.append(Ptime)

    if magnitude == []:
        return

    magnitude = np.array(magnitude)
    amp = np.array(amp)
    distance = np.array(distance)
    depth = np.array(depth)
    Ptimes = np.array(Ptimes)
    RthreePointFivetimes = np.array(RthreePointFivetimes)
    diffs = RthreePointFivetimes - Ptimes

    plt.figure()
    plt.loglog(distance,diffs,'*')
    #plt.xlim([min(magnitudes)-0.5, max(magnitudes)+0.5])
    plt.ylabel('Delay between P and R-wave arrivals [s]')
    plt.xlabel('Distance [m]')
    #plt.ylabel("\Delta T")
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def powerlaw_timedelay_plot(params,data,plotName):
    """@restimates plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param plotName
        name of plot
    """

    index = []
    amp = []
    distance = []
    depth = []
    magnitude = []
    RthreePointFivetimes = []
    Ptimes = []

    for key in data["channels"].iterkeys():
        label = key.replace("_","\_")

        channel_data = data["channels"][key]["powerlaw"]

        for i in range(len(channel_data["index"])):
            index.append(channel_data["index"][i])
            amp.append(channel_data["amp"][i])
            distance.append(channel_data["distance"][i])
            depth.append(channel_data["depth"][i])
            magnitude.append(channel_data["magnitude"][i])

            RthreePointFivetime = channel_data["RthreePointFivetimes"][i]
            Ptime = channel_data["Ptimes"][i]
            RthreePointFivetimes.append(RthreePointFivetime)
            Ptimes.append(Ptime)

    if magnitudes == []:
        return

    index = np.array(index)
    magnitude = np.array(magnitude)
    amp = np.array(amp)
    distance = np.array(distance)
    depth = np.array(depth)
    Ptimes = np.array(Ptimes)
    RthreePointFivetimes = np.array(RthreePointFivetimes)
    diffs = RthreePointFivetimes - Ptimes

    plt.figure()
    fig, ax1 = plt.subplots()
    ax1.loglog(diffs,amp, 'b*')
    ax1.set_xlabel('Delay between P and R-wave arrivals [s]')
    # Make the y-axis label and tick labels match the line color.
    ax1.set_ylabel('amplitude', color='b')
    for tl in ax1.get_yticklabels():
        tl.set_color('b')

    ax2 = ax1.twinx()
    ax2.semilogx(diffs,index, 'r.')
    ax2.set_ylabel('slope', color='r')
    for tl in ax2.get_yticklabels():
        tl.set_color('r')

    #plt.xlim([min(magnitudes)-0.5, max(magnitudes)+0.5])
    #plt.xlabel('Magnitude')
    #plt.ylabel("\Delta T")
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def powerlaw_plot(params,data,plotName):
    """@restimates plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param plotName
        name of plot
    """

    index = []
    amp = []
    distance = []
    depth = []
    magnitude = []

    for key in data["channels"].iterkeys():
        label = key.replace("_","\_")

        channel_data = data["channels"][key]["powerlaw"]

        for i in range(len(channel_data["index"])):
            index.append(channel_data["index"][i])
            amp.append(channel_data["amp"][i]) 
            distance.append(channel_data["distance"][i])
            depth.append(channel_data["depth"][i])
            magnitude.append(channel_data["magnitude"][i])

    if magnitudes == []:
        return

    index = np.array(index)
    magnitude = np.array(magnitude)
    amp = np.array(amp)
    distance = np.array(distance)
    depth = np.array(depth)

    plt.figure()
    fig, ax1 = plt.subplots()
    ax1.loglog(magnitude/distance,amp, 'b*')
    ax1.set_xlabel('Magnitude / Distance')
    # Make the y-axis label and tick labels match the line color.
    ax1.set_ylabel('amplitude', color='b')
    for tl in ax1.get_yticklabels():
        tl.set_color('b')

    ax2 = ax1.twinx()
    ax2.semilogx(magnitude/distance,index, 'r.')
    ax2.set_ylabel('slope', color='r')
    for tl in ax2.get_yticklabels():
        tl.set_color('r')

    #plt.xlim([min(magnitudes)-0.5, max(magnitudes)+0.5])
    #plt.xlabel('Magnitude')
    #plt.ylabel("\Delta T")
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def earthquakes_station(params,data,type,plotName):
    """@earthquakes timeseries plot

    @param params
        seismon params dictionary
    @param data
        list of data structures
    @param plotName
        name of plot
    """

    if len(data["earthquakes_all"]["tt"]) == 0:
        return

    earthquakes_tt = data["earthquakes_all"]["tt"]
    amp = np.log10(data["earthquakes_all"]["data"])
    indexes = np.isinf(amp)
    amp[indexes] = -250
    earthquakes_amp = amp

    threshold = -8

    plot = gwpy.plotter.Plot(figsize=[14,8])
    plot.add_scatter(earthquakes_tt,earthquakes_amp, marker='o', zorder=1000, color='b',label='predicted')

    colors = cm.rainbow(np.linspace(0, 1, len(data["channels"])))
    count=0
    for key in data["channels"].iterkeys():
        label = key.replace("_","\_")

        channel_ttStart = data["channels"][key][type]["ttStart"]
        channel_ttEnd = data["channels"][key][type]["ttEnd"]

        amp = np.log10(data["channels"][key][type]["data"])
        indexes = np.isinf(amp)
        amp[indexes] = -250
        channel_amp = amp

        color = colors[count]
        plot.add_scatter(channel_ttStart,channel_amp,marker='*', zorder=1000, label=label,color=color)
        count=count+1

    xlim = [plot.xlim[0],plot.xlim[1]]
    ylim = [plot.ylim[0],plot.ylim[1]]
    plot.add_line(xlim,[threshold,threshold],label='threshold')

    plot.ylim = [-10,-3]
    #plot.epoch = epoch
    if type == "psd":
        plot.ylabel = 'Mean PSD 0.02-0.1 Hz'
    elif type == "timeseries":
        plot.ylabel = 'Velocity [log10(m/s)]'
    plot.add_legend(loc=1,prop={'size':10}) 

    plot.xlim = [params["gpsStart"],params["gpsEnd"]]

    plot.save(plotName,dpi=200)
    plot.close()

def earthquakes_station_distance(params,data,type,plotName):
    """@earthquakes distance plot

    @param params
        seismon params dictionary
    @param data
        list of data structures
    @param plotName
        name of plot
    """

    plot = gwpy.plotter.Plot(figsize=[14,8])

    colors = cm.rainbow(np.linspace(0, 1, len(data["channels"])))
    count=0
    for key in data["channels"].iterkeys():
        label = key.replace("_","\_")

        channel_data = data["channels"][key]["earthquakes"]

        if len(channel_data["magnitude"]) == 0:
            continue

        color = colors[count]
        vmin = np.min(channel_data["magnitude"])
        vmax = np.max(channel_data["magnitude"])
        kwargs = {'zorder':1000,'label':label,'s':25,'vmin':vmin,'vmax':vmax}
        if type == "time":
            kwargs = {'zorder':1000,'label':label,'s':25,'vmin':vmin,'vmax':vmax}
            plot.add_scatter(channel_data["distance"],channel_data["ttDiff"],c=channel_data["magnitude"],**kwargs)
        elif type == "amplitude":
            kwargs = {'zorder':1000,'label':label,'s':25,'vmin':vmin,'vmax':vmax}
            plot.add_scatter(channel_data["distance"],1e6 * channel_data["ampMax"],c=channel_data["magnitude"],**kwargs) 
        elif type == "residual":
            kwargs = {'zorder':1000,'label':label,'s':25,'vmin':vmin,'vmax':vmax}
            plot.add_scatter(channel_data["distance"],channel_data["residual"],c=channel_data["magnitude"],**kwargs)
        elif type == "velocitymap":
            kwargs = {'zorder':1000,'label':'Velocity map','s':25}
            plot.add_scatter(channel_data["distance"],channel_data["RvelocitymaptimeDiff"],c='r',**kwargs)
            kwargs = {'zorder':1000,'label':'Actual','s':25}
            plot.add_scatter(channel_data["distance"],channel_data["ttDiff"],c='b',**kwargs)
            kwargs = {'zorder':1000,'label':'3.5 km/s','s':25}
            plot.add_scatter(channel_data["distance"],channel_data["distance"]/3500,c='g',**kwargs)
        elif type == "velocitymapresidual":
            kwargs = {'zorder':1000,'label':label,'s':25}
            plot.add_scatter(channel_data["distance"],channel_data["RvelocitymaptimeResidual"],c='k',**kwargs)
        elif type == "lookup":
            kwargs = {'zorder':1000,'label':'Look Up Table','s':25}
            plot.add_scatter(channel_data["distance"],channel_data["RlookuptimeDiff"],c='r',**kwargs)
            kwargs = {'zorder':1000,'label':'Actual','s':25}
            plot.add_scatter(channel_data["distance"],channel_data["ttDiff"],c='b',**kwargs)
            kwargs = {'zorder':1000,'label':'3.5 km/s','s':25}
            plot.add_scatter(channel_data["distance"],channel_data["distance"]/3500,c='g',**kwargs)
        elif type == "lookupresidual":
            kwargs = {'zorder':1000,'label':label,'s':25}
            plot.add_scatter(channel_data["distance"],channel_data["RlookuptimeResidual"],c='k',**kwargs)

        count=count+1

    if count == 0:
        return
    xlim = plot.xlim
    xp = np.linspace(xlim[0],xlim[1],100)

    if type == "time":
        p = np.poly1d([1.0/2000,0])
        label = "prediction [2 km/s]"
        plot.add_line(xp,p(xp),label=label)
        p = np.poly1d([1.0/3500,0])
        label = "prediction [3.5 km/s]"
        plot.add_line(xp,p(xp),label=label)
        p = np.poly1d([1.0/5000,0])
        label = "prediction [5 km/s]"
        plot.add_line(xp,p(xp),label=label)

        plot.ylabel = 'Time [s]'
        plot.axes[0].set_yscale("log")

        distanceMin = 1e6
        distanceMax = 2e7
        timeMin = 1e2
        timeMax = 1e4

        plot.xlim = [distanceMin,distanceMax]
        plot.ylim = [timeMin,timeMax]


    elif type == "amplitude":
        plot.ylabel = r"Velocity [$\mu$m/s]"
        plot.axes[0].set_yscale("log")
    elif type == "residual":
        plot.ylabel = r"Relative difference [(actual-prediction)/prediction]"
        plot.ylim = [get_dist(channel_data["residual"],5)-20,get_dist(channel_data["residual"],90)+20]
    elif type == "velocitymap":
        plot.ylabel = r"Time [s]"
    elif type == "velocitymapresidual":
        plot.ylabel = r"Relative difference [(actual-prediction)/prediction]"
    elif type == "lookup":
        plot.ylabel = r"Time [s]"
    elif type == "lookupresidual":
        plot.ylabel = r"Relative difference [(actual-prediction)/prediction]"

    plot.xlabel = 'Distance [m]'
    plot.add_legend(loc=2,prop={'size':10})

    #plot.add_colorbar(log=False,clim=[vmin,vmax])
    plot.axes[0].set_xscale("log")

    plot.save(plotName,dpi=200)
    plot.close()

def earthquakes_station_distance_heatmap(params,data,type,plotName):
    """@earthquakes distance heatmap plot

    @param params
        seismon params dictionary
    @param data
        list of data structures
    @param plotName
        name of plot
    """

    x = np.array([])
    y = np.array([])

    for key in data["channels"].iterkeys():
        label = key.replace("_","\_")

        channel_data = data["channels"][key]["earthquakes"]

        if len(channel_data["magnitude"]) == 0:
            continue

        x = np.append(x,channel_data["distance"])
        y = np.append(y,channel_data["ttDiff"])

    if type == "time":

        distanceMin = 1e6
        distanceMax = 2e7
        timeMin = 1e2
        timeMax = 1e4
        num=50

        distanceBins = np.logspace(np.log10(distanceMin),np.log10(distanceMax),num=num)
        timeBins = np.logspace(np.log10(timeMin),np.log10(timeMax),num=num)

        bins = np.vstack([distanceBins,timeBins])
        hist2,xedges,yedges = np.histogram2d(x,y, bins=bins, range=None, normed=False, weights=None)

        X,Y = np.meshgrid(distanceBins, timeBins)
        fig = plt.Figure(figsize=[14,8])
        ax = plt.subplot(111)
        im = plt.pcolor(X,Y,hist2.T, cmap=plt.cm.jet)
        ax.set_xscale('log')
        ax.set_yscale('log')
        xp = distanceBins
        linewidth=2.0
        p = np.poly1d([1.0/2000,0])
        label = "prediction [2.0 km/s]"
        plt.loglog(xp,p(xp),'b',label=label,linewidth=linewidth)
        p = np.poly1d([1.0/3500,0])
        label = "prediction [3.5 km/s]"
        plt.loglog(xp,p(xp),'g',label=label,linewidth=linewidth)
        p = np.poly1d([1.0/5000,0])
        label = "prediction [5 km/s]"
        plt.loglog(xp,p(xp),'r',label=label,linewidth=linewidth)

        plt.xlabel('Distance [m]')
        plt.ylabel('Time [s]')
        plt.legend(loc=2,prop={'size':10})
        title = "Earthquake Epicenter Distance to Site vs Arrival Time. %s: %d Earthquakes"%(params["runName"],len(x))
        plt.title(title,fontsize=10)

        plt.xlim([distanceMin,distanceMax])
        plt.ylim([timeMin,timeMax])
        plt.grid(b=True, which='both', color='w',linestyle='--')

        colorbar_label = "Number of events"
        cbar=plt.colorbar()
        cbar.set_label(colorbar_label)

        plt.show()
        plt.savefig(plotName,dpi=200)
        plt.close('all')

def get_dist(array,percentile):

    if len(array) == 0:
        return 0

    array = np.sort(array)
    index = int(np.floor(len(array)*percentile*0.01))
    val = array[index]
    return val

def prediction(data,plotName):
    """@prediction plot

    @param data
        list of data structures
    @param plotName
        name of plot
    """

    if len(data["prediction"]["ttStart"]) == 0:
        return

    prediction_ttStart = data["prediction"]["ttStart"]
    prediction_ttEnd = data["prediction"]["ttEnd"]
    amp = np.log10(data["prediction"]["data"])
    indexes = np.isinf(amp)
    amp[indexes] = -250
    prediction_amp = amp

    threshold = -8

    plot = gwpy.plotter.Plot(figsize=[14,8])
    plot.add_scatter(prediction_ttStart, prediction_amp, marker='o', zorder=1000, color='b',label='predicted')
    #for i in range(len(prediction_ttStart)):
    #    plot.add_line([prediction_ttStart[i],prediction_ttEnd[i]],[prediction_amp[i],prediction_amp[i]],color='b',label='predicted')

    colors = cm.rainbow(np.linspace(0, 1, len(data["channels"])))
    count = 0
    for key in data["channels"].iterkeys():
        label = key.replace("_","\_")

        channel_ttStart = data["channels"][key]["timeseries"]["ttStart"]
        channel_ttEnd = data["channels"][key]["timeseries"]["ttEnd"]

        amp = np.log10(data["channels"][key]["timeseries"]["data"])
        indexes = np.isinf(amp)
        amp[indexes] = -250
        channel_amp = amp

        color = colors[count]
        plot.add_scatter(channel_ttStart,channel_amp,marker='*', zorder=1000, label=label,color=color)
        count=count+1

    xlim = [plot.xlim[0],plot.xlim[1]]
    ylim = [plot.ylim[0],plot.ylim[1]]
    #plot.add_line(xlim,[threshold,threshold],label='threshold')

    plot.ylim = [-10,-4]
    plot.ylabel = 'Velocity [log10(m/s)]'
    plot.add_legend(loc=1,prop={'size':10})

    plot.save(plotName,dpi=200)
    plot.close()

def residual(data,plotName):
    """@residual plot

    @param data
        list of data structures
    @param plotName
        name of plot
    """

    if len(data["prediction"]["ttStart"]) == 0:
        return

    prediction_ttStart = data["prediction"]["ttStart"]
    prediction_ttEnd = data["prediction"]["ttEnd"]
    amp = np.log10(data["prediction"]["data"])
    indexes = np.isinf(amp)
    amp[indexes] = -250
    prediction_amp = amp

    threshold = -8

    indexes = np.where(prediction_amp >= threshold)
    prediction_ttStart = prediction_ttStart[indexes]
    prediction_amp = prediction_amp[indexes]

    plot = gwpy.plotter.Plot(figsize=[14,8])
    plot.add_scatter(prediction_ttStart, prediction_amp, marker='o', zorder=1000, color='b',label='predicted')

    for key in data["channels"].iterkeys():
        label = key.replace("_","\_")

        channel_ttStart = data["channels"][key]["timeseries"]["ttStart"]
        channel_ttEnd = data["channels"][key]["timeseries"]["ttEnd"]

        amp = np.log10(data["channels"][key]["timeseries"]["data"])
        indexes = np.isinf(amp)
        amp[indexes] = -250
        channel_amp = amp

        if len(channel_ttStart) == 0:
            continue

        channel_amp_interp = np.interp(prediction_ttStart,channel_ttStart,channel_amp)
        residual = channel_amp_interp - prediction_amp

        label = "%s residual"%key.replace("_","\_")
        plot.add_scatter(prediction_ttStart,residual,marker='*', zorder=1000, label=label)

    xlim = [plot.xlim[0],plot.xlim[1]]
    ylim = [plot.ylim[0],plot.ylim[1]]
    plot.add_line(xlim,[threshold,threshold],label='threshold')

    #plot.epoch = epoch
    plot.ylabel = 'Velocity [log10(m/s)]'
    plot.add_legend(loc=1,prop={'size':10})

    plot.save(plotName,dpi=200)
    plot.close()

def efficiency(data,plotName):
    """@efficiency plot

    @param data
        list of data structures
    @param plotName
        name of plot
    """

    data = np.array(data)
    t = data[:,0] - data[0,0]

    plt.figure()
    plt.plot(t,np.log10(data[:,2]),marker='*',label="Data")
    plt.plot(t,np.log10(data[:,3]),marker='*',label="Predicted")
    plt.legend(loc=3)
    plt.xlabel('Time [s] [%d]'%data[0,0])
    plt.ylabel('Amplitude')
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def efficiency_limits(data,plotName):
    """@efficiency limits plot

    @param data
        list of data structures
    @param plotName
        name of plot
    """

    data = np.array(data)
    t = data[:,0] - data[0,0]

    plt.figure()
    plt.plot(t,np.log10(data[:,2]),marker='*',label="Data")
    plt.plot(t,np.log10(data[:,3]),marker='*',label="Predicted")
    plt.legend(loc=3)
    plt.ylim([-10,-2])
    plt.xlabel('Time [s] [%d]'%data[0,0])
    plt.ylabel('Amplitude')
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def latencies_sent(params,attributeDics,plotName):
    """@latencies sent plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param plotName
        name of plot
    """

    latencies = []
    for attributeDic in attributeDics:
        latencies.append(attributeDic["SentGPS"]-attributeDic["GPS"])

    if latencies == []:
        return

    plt.figure()
    bins=np.logspace(1,5,15)
    plt.hist(latencies, bins=bins, rwidth=1)
    plt.gca().set_xscale("log")
    plt.gca().set_xlim([10**1,10**5])
    plt.xlabel('Latencies [s]')
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def latencies_written(params,attributeDics,plotName):
    """@latencies written plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param plotName
        name of plot
    """

    latencies = []
    for attributeDic in attributeDics:
        latencies.append(attributeDic["WrittenGPS"]-attributeDic["SentGPS"])

    if latencies == []:
        return

    plt.figure()
    bins=np.linspace(0,100,100)
    plt.hist(latencies, bins=bins, rwidth=1)
    plt.gca().set_xscale("linear")
    plt.gca().set_xlim([0,25])
    plt.xlabel('Latencies [s]')
    #title("Latencies Written")
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def magnitudes(params,attributeDics,plotName):
    """@magnitudes plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param plotName
        name of plot
    """

    gps = []
    magnitudes = []
    for attributeDic in attributeDics:
        gps.append(attributeDic["GPS"])
        magnitudes.append(attributeDic["Magnitude"])

    if magnitudes == []:
        return

    gps = np.array(gps)
    magnitudes = np.array(magnitudes)

    startTime = min(gps)
    #endTime = max(gps)
    endTime = params["gpsEnd"]
    gps = (gps - endTime)/(60)

    plt.figure()
    plt.plot(gps,magnitudes, 'k*')
    plt.ylim([min(magnitudes)-0.5, max(magnitudes)+0.5])
    plt.xlabel('Time [Minutes]')
    plt.ylabel("%.0f - %.0f"%(startTime,endTime))
    plt.xlim([-60, 0])
    plt.title("Magnitude vs. Time")
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def magnitudes_latencies(params,attributeDics,plotName):
    """@magnitudes latencies plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param plotName
        name of plot
    """

    latencies = []
    magnitudes = []
    for attributeDic in attributeDics:
        latencies.append(attributeDic["SentGPS"]-attributeDic["GPS"])
        magnitudes.append(attributeDic["Magnitude"])

    if latencies == []:
        return

    plt.figure()
    plt.plot(latencies, magnitudes, '*')
    plt.gca().set_xscale("log")
    plt.gca().set_xlim([10**1,10**5])
    plt.xlabel('Latencies [s]')
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def variable_magnitudes(attributeDicsDiff,variable,plotName):
    """@variable magnitudes plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param plotName
        name of plot
    """

    variables = []
    magnitudes = []
    for attributeDicDiff in attributeDicsDiff:
        variables.append(attributeDicDiff[variable])
        magnitudes.append(attributeDicDiff["attributeDic2"]["Magnitude"])

    if variables == []:
        return

    plt.figure()
    plt.plot(magnitudes, variables, '*')
    plt.xlabel('Magnitudes')
    plt.xlim([min(magnitudes)-0.5,max(magnitudes)+0.5])
    plt.ylim([min(variables)-0.5,max(variables)+0.5])
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def traveltimes(params,attributeDics,ifo,currentGPS,plotName):
    """@latencies sent plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param ifo
        ifo
    @param currentGPS
        current gps time
    @param plotName
        name of plot
    """

    traveltimes = []

    for attributeDic in attributeDics:

        if not ifo in attributeDic["traveltimes"]:
            continue

        if traveltimes == []:
            arrivalTimes = [max(attributeDic["traveltimes"][ifo]["RthreePointFivetimes"]),max(attributeDic["traveltimes"][ifo]["Stimes"]),max(attributeDic["traveltimes"][ifo]["Ptimes"])]
            traveltimes = np.array(arrivalTimes)
        else:
            arrivalTimes = [max(attributeDic["traveltimes"][ifo]["RthreePointFivetimes"]),max(attributeDic["traveltimes"][ifo]["Stimes"]),max(attributeDic["traveltimes"][ifo]["Ptimes"])]            
            traveltimes = np.vstack([traveltimes,arrivalTimes])

    if traveltimes == []:
        return

    traveltimes = currentGPS-traveltimes

    plotNameSplit = plotName.split("/")
    plotTitle = plotNameSplit[-1].replace(".png","").replace("traveltimes","")

    startTime = np.min(traveltimes)
    endTime = np.max(traveltimes)

    ax = plt.subplot(1,1,1)

    if len(traveltimes.shape) == 1:
        plt.plot(traveltimes[0],1.5 * np.ones(1), 'r*', label='P', markersize=10.0)
        plt.plot(traveltimes[1],2.0 * np.ones(1), 'g*', label='S', markersize=10.0)
        plt.plot(traveltimes[2],2.5 * np.ones(1), 'b*', label='R', markersize=10.0)
        plt.vlines(traveltimes[0], 0, 1.5)
        plt.vlines(traveltimes[1], 0, 2.0)
        plt.vlines(traveltimes[2], 0, 2.5)
    else:
        plt.plot(traveltimes[:,0],1.5 * np.ones(len(traveltimes[:,0])), 'r*', label='P', markersize=10.0)
        plt.plot(traveltimes[:,1],2.0 * np.ones(len(traveltimes[:,1])), 'g*', label='S', markersize=10.0)
        plt.plot(traveltimes[:,2],2.5 * np.ones(len(traveltimes[:,2])), 'b*', label='R', markersize=10.0)
        plt.vlines(traveltimes[:,0], 0, 1.5)
        plt.vlines(traveltimes[:,1], 0, 2.0)
        plt.vlines(traveltimes[:,2], 0, 2.5)
    plt.xlim([startTime-1000, 1000])
    plt.ylim([1, 3])
    plt.xlabel('Countdown [s]')
    plt.title(plotTitle)

    handles, labels = ax.get_legend_handles_labels()
    plt.legend(handles[0:3], labels[0:3])
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def find_nearest(array,value):
    """@find nearest value

    @param array
        array of values
    @param value
        value to compare array to
    """

    array = np.array(array)
    index=(np.abs(array-value)).argmin()
    return array[index], index

def worldmap_plot(params,attributeDics,type,plotName):
    """@worldmap plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param type
        type of worldmap plot
    @param plotName
        name of plot
    """

    ifo = seismon.utils.getIfo(params)

    plt.figure(figsize=(10,5))
    plt.axes([0,0,1,1])

    # lon_0 is central longitude of robinson projection.
    # resolution = 'c' means use crude resolution coastlines.
    m = Basemap(projection='robin',lon_0=0,resolution='c')
    #set a background colour
    m.drawmapboundary(fill_color='#85A6D9')

    # draw coastlines, country boundaries, fill continents.
    m.fillcontinents(color='white',lake_color='#85A6D9')
    m.drawcoastlines(color='#6D5F47', linewidth=.4)
    m.drawcountries(color='#6D5F47', linewidth=.4)

    # draw lat/lon grid lines every 30 degrees.
    m.drawmeridians(np.arange(-180, 180, 30), color='#bbbbbb')
    m.drawparallels(np.arange(-90, 90, 30), color='#bbbbbb')

    if not attributeDics == []:
        attributeDic = attributeDics[0]

        if not "Arbitrary" in attributeDic["traveltimes"]:
            return

        if params["ifo"] == "IRIS":
            distances = []
            for channel in params["channels"]:
                attributeDic = seismon.eqmon.ifotraveltimes_loc(attributeDic, "IRIS", channel.latitude, channel.longitude)
                traveltimes = attributeDic["traveltimes"]["IRIS"]
                distances.append(traveltimes["Distances"][0])
            index = np.argmax(distances)
            channel = params["channels"][index]
            attributeDic = seismon.eqmon.ifotraveltimes_loc(attributeDic, "IRIS", channel.latitude, channel.longitude)
            traveltimes = attributeDic["traveltimes"]["IRIS"]

        else:
            traveltimes = attributeDic["traveltimes"][ifo]

        #traveltimes = attributeDics[0]["traveltimes"][ifo]
        ifolat = traveltimes["Latitudes"]
        ifolng = traveltimes["Longitudes"]
        # compute the native map projection coordinates for cities
        ifox,ifoy = m(ifolng,ifolat)

        m.scatter(
            ifox,
            ifoy,
            s=10, #size
            c='black', #color
            marker='x', #symbol
            alpha=0.5, #transparency
            zorder = 1, #plotting order
            )
        plt.text(
            ifox+50000,
            ifoy+50000,
            ifo,
            color = 'black',
            size='small',
            horizontalalignment='center',
            verticalalignment='center',
            zorder = 2,
            )

    for attributeDic in attributeDics:

        if not "Arbitrary" in attributeDic["traveltimes"]:
            continue

        if params["ifo"] == "IRIS":
            distances = []
            for channel in params["channels"]:
                attributeDic = seismon.eqmon.ifotraveltimes_loc(attributeDic, "IRIS", channel.latitude, channel.longitude)
                traveltimes = attributeDic["traveltimes"]["IRIS"]
                distances.append(traveltimes["Distances"][0])
            index = np.argmax(distances)
            channel = params["channels"][index]
            attributeDic = seismon.eqmon.ifotraveltimes_loc(attributeDic, "IRIS", channel.latitude, channel.longitude)
            traveltimes = attributeDic["traveltimes"]["IRIS"]

        else:
            traveltimes = attributeDic["traveltimes"][ifo]

        if type == "Restimates" and "Restimate" not in traveltimes:
            continue

        x,y = m(attributeDic["Longitude"], attributeDic["Latitude"])
        if type == "Magnitude":
            color = attributeDic["Magnitude"]
            colorbar_label = "Magnitude"
            vmin = 0
            vmax = 7
        elif type == "Traveltimes":
            travel_time = traveltimes["RthreePointFivetimes"][-1] - traveltimes["RthreePointFivetimes"][0]
            travel_time = travel_time / 60
            color = travel_time
            colorbar_label = "Travel times [minutes]"
            vmin = 0
            vmax = 60
        elif type == "Restimates":
            
            travel_time = traveltimes["Restimate"] - traveltimes["RthreePointFivetimes"][0]
            travel_time = travel_time / 60
            color = travel_time
            colorbar_label = "Travel times [minutes]"
            vmin = 0
            vmax = 60
        m.scatter(
                x,
                y,
                s=10, #size
                marker='o', #symbol
                alpha=0.5, #transparency
                zorder = 3, #plotting order
                c=color, 
                vmin=vmin, 
                vmax=vmax
        )

    try:
       cbar=plt.colorbar()
       cbar.set_label(colorbar_label)
       cbar.set_clim(vmin=vmin,vmax=vmax)
    except:
       pass
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def worldmap_channel_plot(params,data,type,plotName):
    """@worldmap plot

    @param params
        seismon params dictionary
    @param data
        list of eqmon structures
    @param type
        type of worldmap plot
    @param plotName
        name of plot
    """

    ifo = seismon.utils.getIfo(params)

    plt.figure(figsize=(10,5))
    plt.axes([0,0,1,1])

    # lon_0 is central longitude of robinson projection.
    # resolution = 'c' means use crude resolution coastlines.
    m = Basemap(projection='robin',lon_0=0,resolution='c')
    #set a background colour
    m.drawmapboundary(fill_color='#85A6D9')

    # draw coastlines, country boundaries, fill continents.
    m.fillcontinents(color='white',lake_color='#85A6D9')
    m.drawcoastlines(color='#6D5F47', linewidth=.4)
    m.drawcountries(color='#6D5F47', linewidth=.4)

    # draw lat/lon grid lines every 30 degrees.
    m.drawmeridians(np.arange(-180, 180, 30), color='#bbbbbb')
    m.drawparallels(np.arange(-90, 90, 30), color='#bbbbbb')

    for key in data["channels"].iterkeys():
        label = key.replace("_","\_")

        channel_data = data["channels"][key]["earthquakes"]

        if len(channel_data["magnitude"]) == 0:
            continue

        x,y = m(channel_data["longitude"], channel_data["latitude"])

        if type == "time":
            travel_time = channel_data["ttDiff"]
            travel_time = travel_time / 60
            color = travel_time
            colorbar_label = "Travel times [minutes]"
            vmin = 0
            vmax = 90

        m.scatter(
                x,
                y,
                s=20, #size
                marker='o', #symbol
                alpha=0.5, #transparency
                zorder = 3, #plotting order
                c=color,
                vmin=vmin,
                vmax=vmax
        )

    try:
       cbar=plt.colorbar()
       cbar.set_label(colorbar_label)
       cbar.set_clim(vmin=vmin,vmax=vmax)
    except:
       pass
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def worldmap_velocitymap(params,plotName):
    """@worldmap plot

    @param params
        seismon params dictionary
    @param plotName
        name of plot
    """

    plt.figure(figsize=(15,10))
    plt.axes([0,0,1,1])

    # lon_0 is central longitude of robinson projection.
    # resolution = 'c' means use crude resolution coastlines.
    m = Basemap(projection='robin',lon_0=0,resolution='c')
    #set a background colour
    m.drawmapboundary(fill_color='#85A6D9')

    # draw coastlines, country boundaries, fill continents.
    m.fillcontinents(color='white',lake_color='#85A6D9')
    m.drawcoastlines(color='#6D5F47', linewidth=.4)
    m.drawcountries(color='#6D5F47', linewidth=.4)

    # draw lat/lon grid lines every 30 degrees.
    m.drawmeridians(np.arange(-180, 180, 30), color='#bbbbbb',zorder=3)
    m.drawparallels(np.arange(-90, 90, 30), color='#bbbbbb',zorder=3)

    velocityFile = '/home/mcoughlin/Seismon/velocity_maps/GR025_1_GDM52.pix'
    velocity_map = np.loadtxt(velocityFile)
    base_velocity = 3.59738

    lats = velocity_map[:,0]
    lons = velocity_map[:,1]
    velocity = 1000 * (1 + 0.01*velocity_map[:,3])*base_velocity

    lats_unique = np.unique(lats)
    lons_unique = np.unique(lons)
    velocity_matrix = np.zeros((len(lats_unique),len(lons_unique)))

    for k in range(len(lats)):
        index1 = np.where(lats[k] == lats_unique)
        index2 = np.where(lons[k] == lons_unique)
        velocity_matrix[index1[0],index2[0]] = velocity[k]

    lons_grid,lats_grid = np.meshgrid(lons_unique,lats_unique)
    x, y = m(lons_grid, lats_grid) # compute map proj coordinates.
    # draw filled contours.

    cs = m.pcolor(x,y,velocity_matrix,alpha=0.5,zorder=2)
    colorbar_label = "Velocity [m/s]"

    try:
       cbar=plt.colorbar()
       cbar.set_label(colorbar_label)
    except:
       pass
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def worldmap_station_plot(params,attributeDics,data,type,plotName):
    """@worldmap plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param data
        channel data dictionary
    @param type
        type of worldmap plot
    @param plotName
        name of plot
    """

    ifo = seismon.utils.getIfo(params)

    plt.figure(figsize=(10,5))
    plt.axes([0,0,1,1])

    # lon_0 is central longitude of robinson projection.
    # resolution = 'c' means use crude resolution coastlines.
    m = Basemap(projection='robin',lon_0=0,resolution='c')
    #set a background colour
    m.drawmapboundary(fill_color='#85A6D9')

    # draw coastlines, country boundaries, fill continents.
    m.fillcontinents(color='white',lake_color='#85A6D9')
    m.drawcoastlines(color='#6D5F47', linewidth=.4)
    m.drawcountries(color='#6D5F47', linewidth=.4)

    # draw lat/lon grid lines every 30 degrees.
    m.drawmeridians(np.arange(-180, 180, 30), color='#bbbbbb')
    m.drawparallels(np.arange(-90, 90, 30), color='#bbbbbb')

    for attributeDic in attributeDics:
        x,y = m(attributeDic["Longitude"], attributeDic["Latitude"])
        m.scatter(
                x,
                y,
                s=20, #size
                marker='x', #symbol
                alpha=0.5, #transparency
                zorder = 3, #plotting order
        )
        plt.text(
                x,
                y+5,
                attributeDic["eventName"],
                color = 'black',
                size='small',
                horizontalalignment='center',
                verticalalignment='center',
                zorder = 3,
        )


    xs = []
    ys = []
    zs = []
    for channel in params["channels"]:
        channel_data = data["channels"][channel.station_underscore]

        if len(channel_data["timeseries"]["data"]) == 0:
            continue

        x,y = m(channel_data["info"].longitude,channel_data["info"].latitude)
        if type == "amplitude":
            z = channel_data["timeseries"]["data"][0] * 1e6
            xs.append(x)
            ys.append(y)
            zs.append(z)
        elif type == "time":
            z = channel_data["timeseries"]["ttMax"][0]
            xs.append(x)
            ys.append(y)
            zs.append(z)

    if xs == []:
        return

    zs = np.array(zs)
    if type == "time":
        if len(attributeDics) == 0:
            minTime = zs[0]
        else:
            minTime = attributeDics[0]["GPS"]
        zs = zs - minTime
        colorbar_label = "dt [s] [%d]"%minTime
        vmin = np.min(zs)
        vmax = np.max(zs)
    elif type == "amplitude":
        #vmin = np.mean(zs) - np.std(zs)
        #vmax = np.mean(zs) + np.std(zs)
        colorbar_label = "Velocity [$\mu$m/s]"
        vmin = np.min(zs)
        vmax = np.max(zs)

    im = m.scatter(
                xs,
                ys,
                s=10, #size
                marker='o', #symbol
                alpha=0.5, #transparency
                zorder = 3, #plotting order
                c=zs
    )

    #try:
    cbar = plt.colorbar(im)
    cbar.set_label(colorbar_label)
    cbar.set_clim(vmin=vmin,vmax=vmax)
    
    #except:
    #   pass
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def worldmap_wavefronts(params,attributeDics,currentGPS,plotName):
    """@worldmap wavefronts plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param currentGPS
        current gps
    @param plotName
        name of plot
    """

    plt.figure(figsize=(10,5))
    plt.axes([0,0,1,1])

    # lon_0 is central longitude of robinson projection.
    # resolution = 'c' means use crude resolution coastlines.
    m = Basemap(projection='robin',lon_0=0,resolution='c')
    #set a background colour
    m.drawmapboundary(fill_color='#85A6D9')

    # draw coastlines, country boundaries, fill continents.
    m.fillcontinents(color='white',lake_color='#85A6D9')
    m.drawcoastlines(color='#6D5F47', linewidth=.4)
    m.drawcountries(color='#6D5F47', linewidth=.4)

    # draw lat/lon grid lines every 30 degrees.
    m.drawmeridians(np.arange(-180, 180, 30), color='#bbbbbb')
    m.drawparallels(np.arange(-90, 90, 30), color='#bbbbbb')

    if not attributeDics == []:
     for ifoName, traveltimes in attributeDics[0]["traveltimes"].items():
        continue
        ifolat = traveltimes["Latitudes"][-1]
        ifolng = traveltimes["Longitudes"][-1]
        # compute the native map projection coordinates for cities
        ifox,ifoy = m(ifolng,ifolat)

        m.scatter(
            ifox,
            ifoy,
            s=10, #size
            c='blue', #color
            marker='o', #symbol
            alpha=0.5, #transparency
            zorder = 2, #plotting order
            )
        plt.text(
            ifox+50000,
            ifoy+50000,
            ifoName,
            color = 'black',
            size='small',
            horizontalalignment='center',
            verticalalignment='center',
            zorder = 3,
            )


    for attributeDic in attributeDics:

        if attributeDic["traveltimes"] == {}:
            continue

        ifoNames = []
        ifoDist = []
        for ifoName, traveltimes in attributeDic["traveltimes"].items():
            ifoDist.append(traveltimes["Distances"][-1])
            ifoNames.append(ifoName)
        ifoIndex = np.array(ifoDist).argmax()
        ifo = ifoNames[ifoIndex]

        nearestGPS, Pindex = find_nearest(attributeDic["traveltimes"][ifo]["Ptimes"],currentGPS)
        Pdist = attributeDic["traveltimes"][ifo]["Distances"][Pindex]/1000
        nearestGPS, Sindex = find_nearest(attributeDic["traveltimes"][ifo]["Stimes"],currentGPS)
        Sdist = attributeDic["traveltimes"][ifo]["Distances"][Sindex]/1000
        nearestGPS, Rindex = find_nearest(attributeDic["traveltimes"][ifo]["Ptimes"],currentGPS)
        Rdist = attributeDic["traveltimes"][ifo]["Distances"][Rindex]/1000

        if currentGPS > max([attributeDic["traveltimes"][ifo]["Ptimes"][-1],attributeDic["traveltimes"][ifo]["Stimes"][-1],attributeDic["traveltimes"][ifo]["RthreePointFivetimes"][-1]]):
            continue

        x,y = m(attributeDic["Longitude"], attributeDic["Latitude"])
        m.scatter(
                x,
                y,
                s=10, #size
                marker='o', #symbol
                alpha=0.5, #transparency
                zorder = 2, #plotting order
        )
        plt.text(
                x,
                y,
                attributeDic["eventName"],
                color = 'black',
                size='small',
                horizontalalignment='center',
                verticalalignment='center',
                zorder = 3,
        )

        X,Y = seismon.eqmon.equi(m, attributeDic["Longitude"], attributeDic["Latitude"], Pdist)
        m.plot(
                X,
                Y,
                linewidth = attributeDic["Magnitude"] / 2,
                zorder = 3, #plotting order
                color = 'b'
        )
        X,Y = seismon.eqmon.equi(m, attributeDic["Longitude"], attributeDic["Latitude"], Sdist)
        m.plot(
                X,
                Y,
                linewidth = attributeDic["Magnitude"] / 2,
                zorder = 3, #plotting order
                color = 'r'
        )
        X,Y = seismon.eqmon.equi(m, attributeDic["Longitude"], attributeDic["Latitude"], Rdist)
        m.plot(
                X,
                Y,
                linewidth = attributeDic["Magnitude"] / 2,
                zorder = 3, #plotting order
                color = 'y'
        )

    plt.show()
    plt.savefig(plotName,dpi=200)
    #savefig(plotNameCounter,dpi=200)
    plt.close('all')

def trip_amp_stage_plot(params,attributeDics,data,type,plotName):
    """@worldmap plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param data
        channel data dictionary
    @param type
        type of worldmap plot
    @param plotName
        name of plot
    """

    trips = []
    platforms = []
    stages = []
    trip_lines = [line.strip() for line in open(params["tripsTextFile"])]
    for line in trip_lines:
        lineSplit = line.split(" ")
        trips.append(float(lineSplit[0]))
        platforms.append(lineSplit[1])
        stages.append(lineSplit[2])
    platforms = set(platforms)
    stages = set(stages)

    plot = gwpy.plotter.Plot(figsize=[14,8])

    colors = cm.rainbow(np.linspace(0, 1, len(platforms)))

    count = 0
    keys = [key for key in data["channels"].iterkeys()]
    #for key in keys:
    for stage in stages:
        xs = []
        ys = []
        for key in keys:
            for platform in platforms:

                if not platform in data["channels"][key]["trips"]:
                    continue
                if not stage in data["channels"][key]["trips"][platform]:
                    continue

                tts = data["channels"][key]["trips"][platform][stage]["gps"]
                velocities = data["channels"][key]["trips"][platform][stage]["velocity"]

                for tt,velocity in zip(tts,velocities):
                    xs.append(tt)
                    ys.append(velocity*1e6)

        color = colors[count]
        plot.add_scatter(xs,ys,marker='*', zorder=1000, label=stage,color=color)
        count = count + 1

    if count == 0:
        return

    xlabel = "time [s]"
    ylabel = "Velocity [$\mu$m/s]"
    plot.axes[0].set_yscale("log")
    #plot.ylim = [1,200]

    plot.xlabel = xlabel
    plot.ylabel = ylabel

    xlim = [plot.xlim[0],plot.xlim[1]]
    ylim = [plot.ylim[0],plot.ylim[1]]
    plot.xlim = xlim
    plot.ylim = [0.1,100]
    plot.add_legend(loc=1,prop={'size':10})

    plot.save(plotName,dpi=200)
    plot.close()

def trip_amp_platform_plot(params,attributeDics,data,type,plotName):
    """@worldmap plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param data
        channel data dictionary
    @param type
        type of worldmap plot
    @param plotName
        name of plot
    """

    trips = []
    platforms = []
    stages = []
    trip_lines = [line.strip() for line in open(params["tripsTextFile"])]
    for line in trip_lines:
        lineSplit = line.split(" ")
        trips.append(float(lineSplit[0]))
        platforms.append(lineSplit[1])
        stages.append(lineSplit[2])
    platforms = set(platforms)
    stages = set(stages)

    plot = gwpy.plotter.Plot(figsize=[14,8])

    colors = cm.rainbow(np.linspace(0, 1, len(platforms)))

    count = 0
    keys = [key for key in data["channels"].iterkeys()]
    #for key in keys:
    for platform in platforms:
        xs = []
        ys = []
        for key in keys:
            for stage in stages:

                if not platform in data["channels"][key]["trips"]:
                    continue
                if not stage in data["channels"][key]["trips"][platform]:
                    continue

                tts = data["channels"][key]["trips"][platform][stage]["gps"]
                velocities = data["channels"][key]["trips"][platform][stage]["velocity"]

                for tt,velocity in zip(tts,velocities):
                    xs.append(tt)
                    ys.append(velocity*1e6)
                
        color = colors[count]
        plot.add_scatter(xs,ys,marker='*', zorder=1000, label=platform,color=color)
        count = count + 1

    if count == 0:
        return

    xlabel = "time [s]"
    ylabel = "Velocity [$\mu$m/s]"
    plot.axes[0].set_yscale("log")
    #plot.ylim = [1,200]

    plot.xlabel = xlabel
    plot.ylabel = ylabel

    xlim = [plot.xlim[0],plot.xlim[1]]
    ylim = [plot.ylim[0],plot.ylim[1]]
    plot.xlim = xlim
    plot.ylim = [0.1,100]
    plot.add_legend(loc=1,prop={'size':10})

    plot.save(plotName,dpi=200)
    plot.close()

def trip_time_plot(params,attributeDics,data,type,plotName):
    """@worldmap plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param data
        channel data dictionary
    @param type
        type of worldmap plot
    @param plotName
        name of plot
    """

    trips = []
    trip_lines = [line.strip() for line in open(params["tripsTextFile"])]
    for line in trip_lines:
        lineSplit = line.split(" ")
        trips.append(float(lineSplit[0]))

    ifo = seismon.utils.getIfo(params)

    plot = gwpy.plotter.Plot(figsize=[14,8])

    xs = []
    ys = []

    count = 0
    keys = [key for key in data["earthquakes"].iterkeys()]
    for key in keys:
        attributeDic = data["earthquakes"][key]["attributeDic"]
        earthquake_data = data["earthquakes"][key]["data"]

        if len(earthquake_data["ampPrediction"]) == 0:
            continue

        if type == "amplitude":
            x = earthquake_data["tt"][0]
            y = earthquake_data["ampMax"][0] * 1e6
        elif type == "time":
            x = earthquake_data["tt"][0]
            y = earthquake_data["ttDiff"][0]

        if y < 2:
            continue

        for trip in trips:
            if np.absolute(x-trip) < 3600:
                print(x, y, trip)

        xs.append(x)
        ys.append(y)

        count = count + 1

    if count == 0:
        return

    plot.add_scatter(xs,ys,marker='o', zorder=1000, color='b',label='predicted')

    if type == "time":
        ylabel = "dt [s]"
    elif type == "amplitude":
        xlabel = "time [s]"
        ylabel = "Velocity [$\mu$m/s]"
        plot.axes[0].set_yscale("log")
        plot.ylim = [1,200]

    #plot.xlabel = xlabel
    plot.ylabel = ylabel

    xlim = [plot.xlim[0],plot.xlim[1]]
    ylim = [plot.ylim[0],plot.ylim[1]]

    for trip in trips:
        kwargs = {"linestyle":"--","color":"k"}
        plot.add_line([trip,trip],ylim,**kwargs)
    plot.xlim = xlim

    plot.save(plotName,dpi=200)
    plot.close()

def trip_plot(params,attributeDics,data,type,plotName):
    """@worldmap plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param data
        channel data dictionary
    @param type
        type of worldmap plot
    @param plotName
        name of plot
    """

    trips = []
    trip_lines = [line.strip() for line in open(params["tripsTextFile"])]
    for line in trip_lines:
        lineSplit = line.split(" ")
        trips.append(float(lineSplit[0]))

    ifo = seismon.utils.getIfo(params)

    plot = gwpy.plotter.Plot(figsize=[14,8])

    xs = []
    ys = []

    count = 0
    keys = [key for key in data["earthquakes"].iterkeys()]
    for key in keys:
        attributeDic = data["earthquakes"][key]["attributeDic"]
        earthquake_data = data["earthquakes"][key]["data"]

        if len(earthquake_data["ampPrediction"]) == 0:
            continue

        if type == "amplitude":
            x = earthquake_data["tt"][0]
            y = earthquake_data["ampMax"][0] * 1e6
        elif type == "time":
            x = earthquake_data["tt"][0]
            y = earthquake_data["ttDiff"][0]

        if y < 2:
            continue

        numtrips = 0
        trip_dist = -1

        for trip in trips:
            if np.absolute(x-trip) < 3600:
                #print(x, y, trip)
                numtrips = numtrips + 1
                trip_dist = x-trip

        print(x, y, numtrips)

        if type == "amplitude": 
            xs.append(y)
            ys.append(numtrips)
        elif type == "time":
            xs.append(y)
            ys.append(trip_dist)

        count = count + 1

    if count == 0:
        return

    plot.add_scatter(xs,ys,marker='o', zorder=1000, color='b',label='predicted')

    if type == "time":
        xlabel = "Velocity [$\mu$m/s]"
        ylabel = "dt [s]"
    elif type == "amplitude":
        xlabel = "Velocity [$\mu$m/s]"
        ylabel = "Number of trips"
        plot.axes[0].set_xscale("log")
        #plot.ylim = [1,200]

    plot.xlabel = xlabel
    plot.ylabel = ylabel

    xlim = [plot.xlim[0],plot.xlim[1]]
    ylim = [plot.ylim[0],plot.ylim[1]]
 
    if type == "amplitude":
        ylim = [-1,6]  

    #for trip in trips:
    #    kwargs = {"linestyle":"--","color":"k"}
    #    plot.add_line([trip,trip],ylim,**kwargs)
    plot.xlim = xlim
    plot.ylim

    plot.save(plotName,dpi=200)
    plot.close()

def station_plot(params,attributeDics,data,type,plotName):
    """@worldmap plot

    @param params
        seismon params dictionary
    @param attributeDics
        list of eqmon structures
    @param data
        channel data dictionary
    @param type
        type of worldmap plot
    @param plotName
        name of plot
    """

    ifo = seismon.utils.getIfo(params)

    plot = gwpy.plotter.Plot(figsize=[14,8])

    count = 0
    keys = [key for key in data["earthquakes"].iterkeys()]
    colors = cm.rainbow(np.linspace(0, 1, len(keys)))
    for key in keys:
        attributeDic = data["earthquakes"][key]["attributeDic"]
        earthquake_data = data["earthquakes"][key]["data"]
        xs = earthquake_data["distance"]

        if type == "amplitude":
            ys = earthquake_data["ampMax"] * 1e6
        elif type == "time":
            ys = earthquake_data["ttDiff"]

        if len(xs) == 0:
            continue

        label = attributeDic["eventName"]
        xs, ys = zip(*sorted(zip(xs, ys)))

        color = colors[count]
        plot.add_scatter(xs,ys,marker='*', zorder=1000, label=label, color=color)
        count=count+1

        z = np.polyfit(xs, ys, 1)
        p = np.poly1d(z)
        xp = np.linspace(np.min(xs),np.max(xs),100)

        label = "%s fit"%attributeDic["eventName"]
        plot.add_line(xp,p(xp),label=label)

        if type == "amplitude":
            c = 18
            fc = 10**(2.3-(attributeDic["Magnitude"]/2.))
            Q = np.max([500,80/np.sqrt(fc)])

            Rfamp = ((attributeDic["Magnitude"]/fc)*0.0035) * np.exp(-2*math.pi*attributeDic["Depth"]*fc/c) * np.exp(-2*math.pi*(xp/1000.0)*(fc/c)*1/Q)/(xp/1000.0)
            Rfamp = Rfamp * 1e6
            label = "%s prediction"%attributeDic["eventName"]
            plot.add_line(xp,Rfamp,label=label)

            Rfamp = 100 + xp * -1e-7*(attributeDic["Magnitude"]/fc) * np.exp(-2*math.pi*attributeDic["Depth"]*fc/c)
            label = "%s linear prediction"%attributeDic["eventName"]
            plot.add_line(xp,Rfamp,label=label)

        elif type == "time":
            p = np.poly1d([1.0/2000,0])
            label = "%s prediction [2 km/s]"%attributeDic["eventName"]
            plot.add_line(xp,p(xp),label=label)
            p = np.poly1d([1.0/3500,0])
            label = "%s prediction [3.5 km/s]"%attributeDic["eventName"]
            plot.add_line(xp,p(xp),label=label)
            p = np.poly1d([1.0/5000,0])
            label = "%s prediction [5 km/s]"%attributeDic["eventName"]
            plot.add_line(xp,p(xp),label=label)

    if count == 0:
        return

    if type == "time":
        ylabel = "dt [s]"
    elif type == "amplitude":
        ylabel = "Velocity [$\mu$m/s]"
        plot.axes[0].set_yscale("log")
        plot.ylim = [1,200]

    plot.xlabel = 'Distance [m]'
    plot.ylabel = ylabel
    plot.add_legend(loc=1,prop={'size':10})
    plot.axes[0].set_xscale("log")

    plot.save(plotName,dpi=200)
    plot.close()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) Duncan Macleod (2016)
#
# This file is part of seismon.
#
# seismon is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# seismon is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with seismon.  If not, see <http://www.gnu.org/licenses/>.

"""Setup the seismon package
"""

# ignore all invalid names (pylint isn't good at looking at executables)
# pylint: disable=invalid-name

from __future__ import print_function

import os, sys
from distutils.version import LooseVersion

from setuptools import (setup, find_packages,
                        __version__ as setuptools_version)

def get_scripts(scripts_dir='bin'):
    """Get relative file paths for all files under the ``scripts_dir``
    """ 
    scripts = []
    for (dirname, _, filenames) in os.walk(scripts_dir):
        scripts.extend([os.path.join(dirname, fn) for fn in filenames])
    return scripts

import versioneer
#from setup_utils import (CMDCLASS, get_setup_requires, get_scripts)
__version__ = versioneer.get_version()
CMDCLASS=versioneer.get_cmdclass()

# -- dependencies -------------------------------------------------------------

# build dependencies
#setup_requires = get_setup_requires()

# package dependencies
install_requires = [
    'arrow',
    'astropy',
    'flask_caching',
    'flask_login',
    'flask_sqlalchemy',
    'flask_wtf',
    'gwpy',
    'lxml',
    'matplotlib>=2.2.0',
    'numpy>=1.7.1',
    'obspy',
    'passlib',
    'redis',
    'scipy>=0.12.1',
    'simplejson',
    'sqlalchemy',
]

# test dependencies
tests_require = [
    'pytest>=3.1',
    'freezegun',
    'sqlparse',
    'bs4',
]
if sys.version < '3':
    tests_require.append('mock')

# -- run setup ----------------------------------------------------------------

setup(
    # metadata
    name='seismon',
    provides=['seismon'],
    version=__version__,
    description="A python package for mitigating the effects of earthquakes on GW detectors",
    long_description=("seismon is a python package for mitigating the effects of earthquakes on GW detectors"),
    author='Michael Coughlin',
    author_email='michael.coughlin@ligo.org',
    license='GPL-2.0-or-later',
    url='https://github.com/gwdetchar/seismon/',

    # package content
    packages=find_packages(),
    scripts=get_scripts(),
    include_package_data=True,
    package_data={'seismon': ['input/*','robustLocklossPredictionPkg/*']},
 
    # dependencies
    cmdclass=CMDCLASS,
    install_requires=install_requires,
    tests_require=tests_require,

    # classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Intended Audience :: Science/Research',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Astronomy',
        'Topic :: Scientific/Engineering :: Physics',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    ],
    extras_require={
        "frontend": [
            'WTForms',
            'WTForms-Alchemy',
            'WTForms-Components'
        ],
    },
)



#!/usr/bin/python

import os, glob, optparse, shutil, warnings, matplotlib, pickle, math, copy, pickle
import matplotlib.pyplot as plt
import numpy as np
import scipy.signal, scipy.stats, scipy.fftpack, scipy.ndimage.filters
from collections import namedtuple
from operator import itemgetter
import seismon.NLNM, seismon.html
import seismon.eqmon

import bokeh.objects, bokeh.glyphs, bokeh.plotting

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__date__ = "2012/8/26"
__version__ = "0.1"

# =============================================================================
#
#                               DEFINITIONS
#
# =============================================================================

def create_plot(x,y,sess):
    source = bokeh.objects.ColumnDataSource(data=dict(x=x, y=y))

    xdr = bokeh.objects.Range1d(start=np.min(x) + 5, end=np.max(x) - 5)
    ydr = bokeh.objects.Range1d(start=np.min(y), end=np.max(y))

    #line = bokeh.glyphs.Line(x="x", y="y", line_color="blue", line_width=2)
    circle = bokeh.glyphs.Circle(x="x", y="y", fill="red", radius=5, line_color="black")
    glyph_renderer = bokeh.objects.GlyphRenderer(
        data_source = source,
        xdata_range = xdr,
        ydata_range = ydr,
        glyph = circle)

    pantool = bokeh.objects.PanTool(dataranges=[xdr, ydr], dimensions=("width","height"))
    zoomtool = bokeh.objects.ZoomTool(dataranges=[xdr,ydr], dimensions=("width","height"))
    resizetool = bokeh.objects.ResizeTool()
    selectiontool = bokeh.objects.SelectionTool()
    previewsavetool = bokeh.objects.PreviewSaveTool()
    overlay = bokeh.objects.BoxSelectionOverlay(tool=selectiontool)

    plot = bokeh.objects.Plot(x_range=xdr, y_range=ydr, data_sources=[source],
        border= 80)
    xaxis = bokeh.objects.LinearAxis(plot=plot, dimension=0, axis_label="X")
    yaxis = bokeh.objects.LinearAxis(plot=plot, dimension=1, axis_label="Y")
    xgrid = bokeh.objects.Rule(plot=plot, dimension=0)
    ygrid = bokeh.objects.Rule(plot=plot, dimension=1)

    plot.renderers.append(glyph_renderer)
    plot.renderers.append(overlay)
    plot.tools = [pantool,zoomtool,resizetool,selectiontool,previewsavetool]

    sess.add(plot, glyph_renderer, xaxis, yaxis, xgrid, ygrid, source, xdr, ydr,
        pantool, zoomtool, resizetool, selectiontool,previewsavetool,overlay)
    sess.plotcontext.children.append(plot)

    return sess

def earthquakes_plot(params,channel,sess):

    if params["ifo"] == "H1":
        ifo = "LHO"
    elif params["ifo"] == "L1":
        ifo = "LLO"
    elif params["ifo"] == "G1":
        ifo = "GEO"
    elif params["ifo"] == "V1":
        ifo = "VIRGO"
    elif params["ifo"] == "C1":
        ifo = "FortyMeter"

    attributeDics = seismon.eqmon.retrieve_earthquakes(params)
    attributeDics = sorted(attributeDics, key=itemgetter("Magnitude"), reverse=True)

    gps = []
    amp = []
    mag = []

    for attributeDic in attributeDics:

        #attributeDic = calculate_traveltimes(attributeDic)

        traveltimes = attributeDic["traveltimes"][ifo]

        gps.append(max(traveltimes["Rtimes"]))
        amp.append(traveltimes["Rfamp"][0])
        mag.append(attributeDic["Magnitude"])

    gps = np.array(gps)
    amp = np.array(amp)
    mag = np.array(mag)

    indexes = np.where(amp > 0)
    gps = gps[indexes]
    amp = amp[indexes]

    gps = (gps - 1056672016.0)/86400.0
    amp = np.log10(amp)

    indexes = np.where(amp > -10)
    gps = gps[indexes]
    amp = amp[indexes]

    sess = create_plot(gps,amp,sess)

    return sess

def psd_plot(params,channel,sess):

    psdLocation = params["dirPath"] + "/Text_Files/PSD/" + channel.station_underscore
    psdLocation = os.path.join(psdLocation,str(params["fftDuration"]))
    files = glob.glob(os.path.join(psdLocation,"*.txt"))

    files = sorted(files)

    ttStart = []
    ttEnd = []
    amp = []

    # Break up entire frequency band into 6 segments
    ff_ave = [1/float(128), 1/float(64),  0.1, 1, 3, 5, 10]

    for file in files:

        fileSplit = file.split("/")
        txtFile = fileSplit[-1].replace(".txt","")
        txtFileSplit = txtFile.split("-")
        thisTTStart = int(txtFileSplit[0])
        thisTTEnd = int(txtFileSplit[1])

        if (thisTTStart < params["gpsStart"]) or (thisTTEnd > params["gpsEnd"]):
            continue

        ttStart.append(thisTTStart)
        ttEnd.append(thisTTEnd)

        data = np.loadtxt(file)
        thisSpectra = data[:,1]
        thisFreq = data[:,0]

        freqAmps = []

        for i in range(len(ff_ave)-1):
            newSpectraNow = []
            for j in range(len(thisFreq)):
                if ff_ave[i] <= thisFreq[j] and thisFreq[j] <= ff_ave[i+1]:
                    newSpectraNow.append(thisSpectra[j])
            freqAmps.append(np.mean(newSpectraNow))

        thisAmp = freqAmps[1]
        amp.append(thisAmp)

    ttStart = np.array(ttStart)
    ttEnd = np.array(ttEnd)
    amp = np.array(amp)

    gps = (ttStart - 1056672016.0) / 86400.0
    amp = np.log10(amp)

    sess = create_plot(gps,amp,sess)

    return sess
 
def channel_page(params, channel):

    plotLocation = params["path"] + "/" + channel.station_underscore
    if not os.path.isdir(plotLocation):
        os.makedirs(plotLocation)

    outputFile = os.path.join(plotLocation,"bokeh.html")

    sess = bokeh.session.HTMLFileSession(outputFile)
    sess = earthquakes_plot(params,channel,sess)
    sess = psd_plot(params,channel,sess)
    sess.save(js="relative", css="relative", rootdir=os.path.abspath("."))
    print("Wrote", outputFile)


